/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

// Globals
function variableTreeView(stackViewer, context) {
  treeView.apply(this, []);
  this.mSingleSelect = true;
  this.mVariableList = [];
  this.mFrameVarList = [];
  this.context = context;
  this.debug=0;
  this.stackViewer = stackViewer;
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
variableTreeView.prototype = new treeView();
variableTreeView.prototype.constructor = variableTreeView;

variableTreeView.prototype.setContext = function(context) {
    this.context = context;
}

variableTreeView.prototype.haveVariable = function(expression) {
    try {
        this.stackViewer.stackstate.lookupDebugProperty(this.context, expression);
    } catch (e) {
        //dump("variableTreeView.haveVariable("+expression+")=false\n");
        return false;
    }
    //dump("variableTreeView.haveVariable("+expression+")= true\n");
    return true;
}

variableTreeView.prototype.addVariable = function(expression, value) {
    //dump("variableTreeView.addVariable("+expression+", "+value+")\n");
    // unable to set the value, so we add a new value
    this.stackViewer.addWatchedVar(expression);
    if (value && typeof(value)!="undefined") {
        //dump("variableTreeView.setVariable attempt to new var value\n");
        this.stackViewer.valueChanged(this.context, expression, value);
    }
}

variableTreeView.prototype.setVariable = function(index, expression, value) {
    //dump("variableTreeView.setVariable("+expression+", "+value+")\n");
    //this.stackViewer.valueChanged(this.context, expression, value);
    var variable = this.mVariableList[index];
    if (variable.fullname != expression) {
        // they've changed the name on us, add a new variable
        this.addVariable(expression,value);
    }
    
    var exceptionText = null;
    try {
        var got_error = new Object();
        variable.debugprop.SetValueAsString(value, got_error);
        exceptionText = got_error.value;
    } catch (e) {
        exceptionText = "unexpected error setting value";
    }
    variable.value = null;
    if (exceptionText) {
        alert("Error setting value: " + exceptionText);
    }
    this.stackViewer.update_for_stack_change();
}

variableTreeView.prototype.removeSelectedVariables = function() {
    for (var index=0; index < this.mTotalRows; index++) {
        if (this.mSelection.isSelected(index)) {
            var variable = this.mVariableList[index];
            if (variable.level == 0) {
                this.stackViewer.stackstate.removeWatchedVarName(variable.fullname);
                if (this.stackViewer.current_frame) {
                    this.stackViewer.stackstate.removeDebugProperty(this.context, variable.fullname);
                }
            }
        }
    }
    this.stackViewer.update_watched_variables();
}

variableTreeView.prototype.removeVariable = function(index) {
    // we can only remove watched variables, so assume that is what we want to do
    if (this.mVariableList[index].level == 0) {
        this.stackViewer.removeWatchedVar(this.mVariableList[index].fullname);
    }
}

variableTreeView.prototype.getName = function(index) {
    //dump("variableTreeView.getName "+this.mVariableList[index].fullname+"\n");
    if (index < this.mVariableList.length) {
        var variable = this.mVariableList[index];
        if (!variable.fullname) {
            variable.fullname = variable.debugprop.fullName;
        }
        return variable.fullname;
    }
    return null;
}

variableTreeView.prototype.getValue = function(index) {
    //dump("variableTreeView.getValue "+this.mVariableList[index].value+"\n");
    if (index < this.mVariableList.length) {
        var variable = this.mVariableList[index];
        if (!variable.value) {
            variable.value = variable.debugprop.value;
        }
        return variable.value;
    }
    return null;
}

variableTreeView.prototype.clearList = function() {
    //dump("variableTreeView.clearList\n");
    this.mVariableList = [];
    this.mFrameVarList = [];
    this.mTotalRows = 0;
    if (this.mTree) {
        this.mTree.rowCountChanged(0,0);
        this.mTree.invalidate();
    }
}

variableTreeView.prototype._getChildren = function (debugprop) {
    try {
        var children = new Object();
        debugprop.enumChildren(children, new Object());
        return children.value;
    } catch(e) { /* do nothing */ }
    return null;
}

variableTreeView.prototype.dumpVariable = function(v,i) {
    if (!v.debugprop) 
        dump("var: i: "+i+" pindex: "+v.parent+" level: "+v.level+" havechild: "+v.isContainer+"\n");
    else
        dump("var: i: "+i+" pindex: "+v.parent+" level: "+v.level+" havechild: "+v.isContainer+" name: "+v.debugprop.fullName+"\n");
}

variableTreeView.prototype.makeVariable = function(debugprop, parent, level, nameonly) {
    var variable = new Object();
    var children = null;
    var placeholder = 0;
    variable.isEmpty = false;
    variable.parent = parent;
    variable.level = level;
    variable.isContainerOpen = false;
    variable.children = null;
    variable.fullname = null;
    variable.name = null;
    if (nameonly) {
        variable.fullname = debugprop;
        variable.name = debugprop;
        variable.value = "";
    } else {
        variable.debugprop = debugprop;
        variable.isContainer = variable.debugprop.hasChildren;
        if (variable.isContainer) {
            variable.fullname = variable.debugprop.fullName;
            variable.isContainerOpen = this.stackViewer.stackstate.isVarExpanded(this.context, variable.fullname);
        }
    }
    return variable;
}

variableTreeView.prototype.getVariables = function(debugpropArray, parent, level, nameonly) {
    var newArray = [];
    for (var i =0; i < debugpropArray.length; i++) {
        var variable = this.makeVariable(debugpropArray[i], parent, level, nameonly);
        newArray[newArray.length] = variable;
    }
    return newArray;
}

variableTreeView.prototype.buildList = function(debugpropArray, frameIndex) {
    window.setCursor("wait");
    this.mVariableList = this.getVariables(debugpropArray, -1, 0, frameIndex == -1);
    this.mTotalRows = this.mVariableList.length;
    //dump("variableTreeView.buildList length "+this.mVariableList.length+", got "+debugpropArray.length+"\n");
    if (this.mTree) {
        this.mTree.rowCountChanged(0,this.mVariableList.length);
        this.mTree.invalidate();
    }
    window.setCursor("auto");
}

variableTreeView.prototype.getCellText = function(row, colID) {
    var res = "";
    if (row < this.mVariableList.length) {
        var variable = this.mVariableList[row];
        try {
            if (colID == "name") {
                if (!variable.name) {
                    variable.name = variable.debugprop.name;
                }
                // yes, we still want to retreive fullname here
                if (!variable.fullname) {
                    variable.fullname = variable.debugprop.fullName;
                }
                res = variable.name;
            } else if (colID == "value") {
                if (typeof(variable.value) == "undefined") {
                    variable.value = variable.debugprop.value;
                }
                res = variable.value;
            }
        } catch(e) {
            // we access the stackviewer layer (debugprop),
            // which when not debugging, will
            // throw exceptions
        }
    }
    //dump("variableTreeView.getCellText("+row+", "+colID+")=["+res+"]\n");
    return res;
}

variableTreeView.prototype.isEditable = function(row, colID) {
    //dump("variableTreeView.isEditable("+row+", "+colID+")\n");
    if (colID == "value") return true;
    return false;
}

variableTreeView.prototype.isContainer = function(index) {
    var res = 0;
    if (index < this.mVariableList.length && typeof(this.mVariableList[index]) != 'undefined') {
        res = this.mVariableList[index].isContainer;
    }
    //dump("variableTreeView.isContainer("+index+")=["+res+"]\n");
    return res;
}

variableTreeView.prototype.isContainerOpen = function(index) {
    var res = false;
    if (index < this.mVariableList.length && typeof(this.mVariableList[index]) != 'undefined') {
        var variable = this.mVariableList[index];
        res = variable.isContainerOpen;
        if (res && ! variable.children) {
            this._addChildren(index, variable);
        }
    }
    //dump("variableTreeView.isContainerOpen("+index+")=["+res+"]\n");
    return res;
}

  /* long getLevel(in long index); */
variableTreeView.prototype.getLevel = function(index) {
    //dump("variableTreeView.getLevel("+index+")=["+this.mVariableList[index].level+"]\n");
    if (index < this.mVariableList.length && typeof(this.mVariableList[index]) != 'undefined') {
        return this.mVariableList[index].level;
    }
    return 0;
}

  /* boolean hasNextSibling(in long rowIndex, in long afterIndex); */
variableTreeView.prototype.hasNextSibling = function(rowIndex, afterIndex) {
    if (afterIndex < (this.mTotalRows - 1) && typeof(this.mVariableList[rowIndex]) != 'undefined') {
        var variable = this.mVariableList[rowIndex];
        for (var i = rowIndex + 1; i < this.mTotalRows && variable.level <= this.mVariableList[i].level; i++) {
            if (variable.level == this.mVariableList[i].level) return true;
        }
    }
    return false;
}

  /* long getParentIndex(in long rowIndex); */
variableTreeView.prototype.getParentIndex = function(index) {
    var parent = -1;
    if (index < this.mVariableList.length && index > 0 && this.mVariableList[index].level > 0) {
        // parent is not going to be correct most of the time, go up the list and compare levels
        var level = this.mVariableList[index].level;
        for (var i = index - 1; i >= 0; i--) {
            if (level > this.mVariableList[i].level) {
                parent = i;
                break;
            }
        }
    }
    return parent;
}

variableTreeView.prototype._addChildren = function(index, variable) {
    window.setCursor("wait");
    var children = this._getChildren(variable.debugprop);
    if (!children) return;
    variable.children = this.getVariables(children, index, variable.level+1);

    var fa = this.mVariableList.slice(0,index+1);
    var fe = this.mVariableList.slice(index+1);
    this.mVariableList = fa;
    this.mVariableList = this.mVariableList.concat(variable.children);
    this.mVariableList = this.mVariableList.concat(fe);
    this.mTotalRows = this.mVariableList.length;
    this.mTree.rowCountChanged(0,this.mVariableList.length);
    window.setCursor("auto");
    this.mTree.invalidate();
}

/* this function is *slower than it should be* if dealing with *deeply* nested structues
    where a lot of them are toggled open because we're splitting the array and recreating
    it on every recursion. */
variableTreeView.prototype._toggleAddChildren = function(index) {
    var variable = this.mVariableList[index];
    if (!variable.children) {
        // children have not been retreived, get them now
        var children = this._getChildren(variable.debugprop);
        if (!children) return;
        variable.children = this.getVariables(children, index, variable.level+1);
    }
    
    // open the container
    var fa = this.mVariableList.slice(0,index+1);
    var fe = this.mVariableList.slice(index+1);

    this.mVariableList = fa;

    // insert the children into the array
    this.mVariableList = this.mVariableList.concat(variable.children);
    if (fe.length > 0) this.mVariableList = this.mVariableList.concat(fe);
    
    // now we have a new index, check if any of the new children are open
    for (var i = index + 1; i < this.mVariableList.length && variable.level < this.mVariableList[i].level; i++) {
        if (this.mVariableList[i].level == variable.level + 1 && this.mVariableList[i].isContainerOpen) {
            this._toggleAddChildren(i);
        }
    }
}

  /* void toggleOpenState(in long index); */
variableTreeView.prototype.toggleOpenState = function(index) {
    if (index >= this.mVariableList.length) return;
    var variable = this.mVariableList[index];
    if (variable.isContainer) {
        window.setCursor("wait");
        var firstVisibleRow = this.mTree.getFirstVisibleRow();
        if (variable.isContainerOpen) {
            // close the container
            this.stackViewer.stackstate.contractVar(this.context, variable.fullname);
            variable.isContainerOpen = false;
            var level = variable.level;
            var i;
            for (i = index + 1; i < this.mVariableList.length && level < this.mVariableList[i].level; i++) { }
            this.mVariableList.splice(index+1,i-(index+1));
        } else {
            this.stackViewer.stackstate.expandVar(this.context, variable.fullname);
            variable.isContainerOpen = true;
            this._toggleAddChildren(index);
        }
        this.mTotalRows = this.mVariableList.length;
        this.mTree.rowCountChanged(0,this.mVariableList.length);
        this.mTree.ensureRowIsVisible(firstVisibleRow);
        this.mTree.invalidate();
        //dump("variableTreeView.toggleOpenState length "+this.mVariableList.length+"\n");
        window.setCursor("auto");
    }
}

variableTreeView.prototype.Sort = function(colId) {
    // do nothing for now
}
