/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/*
 * This file contains the functions that launch new windows such as
 * the help system.
 *
 */

var gHelpArg_StartPage = null;
var gHelpArg_Align;

function _launch_HelpWithRefresh()
{
    const nsIWindowMediator = Components.interfaces.nsIWindowMediator;
    const MEDIATOR_CONTRACTID = "@mozilla.org/appshell/window-mediator;1";
    var windowManager = Components.classes[MEDIATOR_CONTRACTID].getService(nsIWindowMediator);
    var helpWindow = windowManager.getMostRecentWindow('komodo_help');
    if (helpWindow) {
        // XXX it'd be nice to have a way to change the help page of an already
        // open window instead of closing and restarting it.
        helpWindow.close();
    }

    helpWindow = openWindowUniqueInstance("chrome://jaguar/content/help/help.xul",
                             'komodo_help',
                             'chrome,all,resizable,scrollbars,dependent=true,dialog=no');
}

function launch_MainHelp(page) {
    if (typeof(page) == "undefined") page = null;
    if (page) {
        gHelpArg_StartPage = page;
    }
    gHelpArg_Align = false;
    _launch_HelpWithRefresh();
}

function launch_Tutorials(tutorial /* =null */)
{
    // Open the Tutorial in the help browser and align the two windows side
    // by side.
    //    "tutorial" can optionally be used to specify a particular tutorial
    //        to start with. Currently valid values are "perl", "python", 
    //        "php", "xslt", and null (to go to Tutorial home page).
    if (tutorial) {
        gHelpArg_StartPage = "chrome://jaguar/content/doc/komodo/tutorial/"+
                             tutorial+"tut/"+tutorial+"tut.html";
    } else {
        gHelpArg_StartPage = "chrome://jaguar/content/doc/komodo/tutorial/komtutorials.html";
    }
    gHelpArg_Align = true;
    _launch_HelpWithRefresh();

}

function launch_TipOfTheDay() {
    openWindowUniqueInstance("chrome://jaguar/content/tipoftheday/tipoftheday.xul",
                "komodo_tip",
                "chrome,dialog,dependent");
}

function launch_LanguageHelp() {
    // Get the current document's language.
    var language = null;
    var view = gViewMgr.currentView;
    if (view != null) {
        language = view.document.language;
    }

    // Get the help command appropriate for that language.
    var command=null, name=null;
    if (language && gPrefs.hasStringPref(language+'HelpCommand')) {
        command = gPrefs.getStringPref(language+'HelpCommand');
        if (command) {
            name = language + " Help";
        }
    }
    if (!command) {
        // Couldn't find language-specific help command -- use the default one.
        command = gPrefs.getStringPref('DefaultHelpCommand');
        name = "Help";
    }

    Run_RunCommand(window,
                   command,
                   null, // cwd
                   null, // env 
                   false, // insertOutput
                   false, // operateOnSelection
                   true, // doNotOpenOutputWindow
                   'no-console', // runIn
                   0, // parseOutput
                   '', // parseRegex
                   0, // showParsedOutputList
                   name); // name
}

function launch_AlternateLanguageHelp() {
    var command = gPrefs.getStringPref('OtherHelpCommand');
    Run_RunCommand(window,
                   command,
                   null, // cwd
                   null, // env 
                   false, // insertOutput
                   false, // operateOnSelection
                   true, // doNotOpenOutputWindow
                   'no-console', // runIn
                   0, // parseOutput
                   '', // parseRegex
                   0, // showParsedOutputList
                   "Alternate Help"); // name
}

function launch_Find() {
    _launch_FindTab("find");
}

function launch_Replace() {
    _launch_FindTab("replace");
}

var gFindDialogPanel = null;
function _launch_FindTab(panel) {
    // Transfer focus to the hidden input buffer to capture keystrokes from the
    // user while find.xul is loading. The find dialog will retrieve these
    // contents when it is ready.
    InputBuffer_Start();

    gFindDialogPanel = panel; // special global to pass info to find.xul
    window.openDialog("chrome://jaguar/content/find/find.xul",
                      "komodo_find",
                      "chrome,dialog,dependent,close=yes");
}

function launch_RunCommand() {
    // Transfer focus to the hidden input buffer to capture keystrokes from the
    // user while run.xul is loading.  (Get the current view before calling
    // inputbuffer start so we have the correct focus coming out of the
    // dialog.)
    InputBuffer_Start();
    openWindowUniqueInstance("chrome://jaguar/content/run/run.xul", 
                             "komodo_run", 
                             "chrome,dialog,dependent");
}

function launch_DiffWindow(text, title /* ="Diff Output" */)
{
    if (typeof(title) == "undefined") {
        title = "Diff Output";
    }
    var obj = new Object();
    obj.title = title;
    obj.text = text;
    obj.language = 'Diff';
    obj.dimensionPrefPrefix = "sccDiffWin-";
    obj.processDiffs = true;
    openWindowUniqueInstance("chrome://jaguar/content/simpleeditor/simpleeditor.xul", 
                             "diff-window", 
                             "chrome,dialog=no,resizable=yes,scrollbars,close=yes",
                             obj);
}

function launch_WSBookmarks()  {
    openWindowUniqueInstance("chrome://jaguar/content/webservices/bookmarks.xul", 
                             "komodo_wsbookmarks",
                             "chrome,dialog,dependent");
}    

function launch_watchLocalFile() {
    var filename = filepicker_openFile();
    if (filename)
        openWindowMultipleInstance("chrome://jaguar/content/tail/tail.xul",
                                            "komodo_tail",
                                            "chrome,resizable,scrollbars,dialog=no",
                                            filename,
                                            window);
}

function launch_rxWindow() {
    openWindowUniqueInstance('chrome://jaguar/content/rx/koRx.xul',
                                'komodo_rx',
                                'chrome,resizable,dialog=no,close=yes,dependent=no',
                                gViewMgr.currentView);
}

function launch_perlappWindow() {
    var sysDataSvc = Components.classes["@activestate.com/koSystemDataService;1"]
                     .getService(Components.interfaces.koISystemDataService);
        
    var versionStr = parseFloat(sysDataSvc.getString("pdk.version"));
    openWindowUniqueInstance("chrome://jaguar/content/pdk/perlapp.xul",
                "komodo_perlapp",
                "chrome,resizable,dialog=no",
                versionStr);
}


























//---- Input buffering
// When you need to capture user input while a slow XUL window is loading you
// can use the input buffer. Usage:
//  - in some JS code:
//      InputBuffer_Start()
//      // open XUL window
//
//  - in slow XUL window onload handler:
//      var contents = InputBuffer_Finish();
//      // use the contents somehow
//

var _gInputBuffer_active;

function InputBuffer_Start()
{
    _gInputBuffer_active = 1;
    var inputBufferWidget = document.getElementById("hidden-input-buffer");
    inputBufferWidget.focus();
}

function InputBuffer_OnFocus(event)
{
    // if it is not active the hidden input buffer should not have the focus
    if (event && event.eventPhase == event.BUBBLING_PHASE &&
        !_gInputBuffer_active && gViewMgr.currentView) {
        // This has to be in a timeout for the controllers to work right.
        window.setTimeout('gViewMgr.currentView.setFocus();', 1)
    }
}


function InputBuffer_Finish()
{
    // Return the contents of the input buffer and stop buffering.
    var inputBufferWidget = document.getElementById("hidden-input-buffer");
    var contents = inputBufferWidget.value;
    inputBufferWidget.value = "";

    _gInputBuffer_active = 0;
    return contents;
}


