/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/* Komodo command handling.
   This file encapsulates some of the magic we do w.r.t. command handling.
   I'm not sure I don't want to get rid of some of this, but we'll see.
   
   There are two public functions:
   
        command_doCommand(command)
    
    and
        command_updateCommandset(commandset)

 */

const _keepUpdateTimes = false;

var commandLog = null;
function command_onload() {
    commandLog = getLoggingMgr().getLogger('commands');
    //commandLog.setLevel(LOG_DEBUG);
}

// Returns "true" is the command is a "checkbox" type command - ie,
// if supported, the "isCommandEnabled()" function determines the check-state,
// rather than the grayness of the command itself.
function _command_isCheckCommand(id)
{
    var node = top.document.getElementById(id);
    if (node) {
        var t = node.getAttribute("type").toLowerCase();
        if (t == "checkbox" || t == "radio") {
            return true;
        } else {
            return node.hasAttribute("toggled");
        }
    }
    return false;
}

// Returns true if the command is enabled or checked.
function _command_isCommandEnabled(id) {
    var node = top.document.getElementById(id);
    if (node) {
        if (_command_isCheckCommand(id)) {
            return node.getAttribute("checked")
        }
        return !node.getAttribute("disabled")
    }
    commandLog.warn("isCommandEnabled - no ID '" + id + "'");
    return false;
}

// Returns true if the command was handled (even if it is disabled),
// or false if no handler can be found, or an error occurs.
function _command_updateCommand(command, controller /* = null */) {
//    gTimeline.startTimer('    updating command: ' + command);
    try {
        var found = false;
        if (controller == undefined || controller == null) {
            controller = _command_getControllerForCommand(command);
        }
  
        var enabled = false;
        if (controller) {
            enabled = controller.isCommandEnabled(command);
            found = true;
        }
        commandLog.debug("Command: " + command + " is set to " + enabled)
        _command_setCommandEnabled(command, found, enabled);
    } catch (e) {
        commandLog.exception(e,"doing _command_updateCommand: " + command + ': ');
    }
//    gTimeline.stopTimer('    updating command: ' + command);
//    gTimeline.markTimer('    updating command: ' + command);
    return found;
}

function command_updateCommandset(commandset) {
//    gTimeline.startTimer('updating commandset: ' + commandset.id);
    commandLog.info('command_updateCommandset: ' + commandset.id);
    var childNodes = commandset.childNodes;
    var length = childNodes.length;
    for (var i = 0; i < length; i++) {
        var commandID = childNodes[i].id;
        if (commandID) {
            if (!_command_updateCommand(commandID)) {
                commandLog.debug("command_updateCommandset - " + commandset.id +
                          " - command ID '" + commandID + "' - no controller available!");
            }
        } else {
            commandLog.warn("command_updateCommandset - " + commandset.id + " - element no " + i + " has no ID!!");
        }
    }
//    gTimeline.stopTimer('updating commandset: ' + commandset.id);
//    gTimeline.markTimer('updating commandset: ' + commandset.id);
}

function _command_getControllerForCommand(command) {
    var commandDispatcher = top.document.commandDispatcher
    var focusedElement = commandDispatcher.focusedElement;
    var controller;
    
    // If html:embed tag, *must* go directly to its controller.
    // Can not use the commandDispatcher first, as some builtin commands
    // will always be satisfied, and never defer to the plugin.
    // this works in combination with our plugin events patch, which
    // must be present for the plugin to get key events in the first place
    if (focusedElement && focusedElement.parentNode &&
        (focusedElement.tagName == "html:embed")) {
        controller = focusedElement.parentNode.controllers.getControllerForCommand(command);
    }
    
    if (!controller)
        controller = commandDispatcher.getControllerForCommand(command);
    return controller
}


function _command_setCommandEnabled(id, supported, enabled)
{
    var node = top.document.getElementById(id);

    if ( node ) {
        var really_enabled = supported;
        if (supported && _command_isCheckCommand(id)) {
            if ( enabled ) {
                node.setAttribute('checked', 'true');
                node.setAttribute('toggled', 'true');
            }
            else {
                node.setAttribute('checked', 'false');
                node.setAttribute('toggled', 'false');
            }
        } else {
            really_enabled = enabled;
        }
        if (really_enabled)
            node.removeAttribute("disabled");
        else
            node.setAttribute('disabled', 'true');
    } else {
	commandLog.warn("Attempting to update command '" + id + "', but no node available");
    }
}

function koRequestUpdateCommands(event) {
    window.updateCommands(event);
    commandLog.warn("using old koRequestUpdateCommands: " + event);
}

// Execute a command.
function command_doCommand(command) {
    commandLog.debug("command_doCommand(" + command + ")");
    var rc = false;
    try {
        var controller = _command_getControllerForCommand(command);
        if (controller) {
            // gMacroMgr can be undefined if we're not in the main komodo.xul window (e.g. Rx commands)
            if (typeof(gMacroMgr) != 'undefined' && gMacroMgr && gMacroMgr.recording) {
                if (! document.getElementById(command).hasAttribute('interactive')) {
                    gMacroMgr.appendCommand(command);
                }
            }
            if (_command_isCheckCommand(command)) {
                // For a check command, we do the command, then re-update it,
                // (as presumably the state has now toggled)
                controller.doCommand(command);
                _command_updateCommand(command, controller)
            } else {
                controller.doCommand(command);
            }
            rc = true;
        }
        else {
            commandLog.warn("Command " + command + " has no controller.");
        }
    }
    catch (e) {
        commandLog.exception(e,"An error occurred executing the "+command+" command");
        throw Components.results.NS_ERROR_FAILURE;
    }
    return rc;
}

var _originalUpdateCommands = window.updateCommands;
function myUpdateCommands(event) {
    commandLog.info("Doing updateCommands: '" + event + "'");
    _originalUpdateCommands(event);
}
window.updateCommands = myUpdateCommands;
    
