/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/* ---- UI Helpers for showing/hiding, expanding collapsing, various UI features ---- */

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var _uilayout_log = getLoggingMgr().getLogger('uilayout');

// Toggle the visibility of the specified toolbar,
// along with the corresponding broadcaster if it exists.
function uilayout_toggleToolbarVisibility(toolbarId)
{
    var toolbar = document.getElementById(toolbarId);
    if (!toolbar) {
        _uilayout_log.error("Couldn't find toolbar: " + toolbarId);
        return;
    }
    var broadcasterId = toolbar.getAttribute('broadcaster');
    if (!broadcasterId) {
        _uilayout_log.info("No broadcaster associated with toolbar: " + toolbarId);
        return;
    }
    var broadcaster = document.getElementById(broadcasterId);
    if (!broadcaster) {
        _uilayout_log.error("Couldn't find broadcaster: " + broadcasterId);
        return;
    }
    if (toolbar.hidden) {
        toolbar.setAttribute("hidden", "false");
        broadcaster.setAttribute("checked", "true");
    } else {
        toolbar.setAttribute("hidden", "true");
        broadcaster.setAttribute("checked", "false");
    }
}

// 'toolbarId' is the id of the toolbar that should be affected
// 'show' is a boolean -- true means show the text.
function _uilayout_setToolbarButtonText(toolbarId, buttonTextShowing)
{
    var toolbar = document.getElementById(toolbarId);
    if (!toolbar) {
        _uilayout_log.error("Could not find toolbar with id: " + toolbarId);
    }
    try {
        if (buttonTextShowing) {
            toolbar.removeAttribute('buttonstyle');
            toolbar.update('toolbarbutton',null);
        } else {
            toolbar.setAttribute('buttonstyle','pictures');
            toolbar.update('toolbarbutton','pictures');
        }
    } catch(e) {
        _uilayout_log.error(e);
    }
}

// Toggle whether text is shown on toolbar buttons
function uilayout_toggleButtons()
{
    var buttonTextShowing;
    var broadcaster = document.getElementById('cmd_toggleButtonText');
    if (broadcaster.hasAttribute('checked') && broadcaster.getAttribute('checked') == 'true') {
        broadcaster.setAttribute("checked", "false");
        buttonTextShowing = false;
    } else {
        broadcaster.setAttribute("checked", "true");
        buttonTextShowing = true;
    }
    _uilayout_setToolbarButtonText('standardToolbar', buttonTextShowing);
    _uilayout_setToolbarButtonText('debuggerToolbar', buttonTextShowing);



    uilayout_updateToolbarArrangement(buttonTextShowing);
    document.persist('cmd_toggleButtonText', 'checked');
}

function uilayout_updateToolbarArrangement(buttonTextShowing /* default: look it up */)
{
    var menuItem = document.getElementById('menu_toggleButtonText');
    var fromloading = false;
    var contextMenuItem = document.getElementById('menu_toggleButtonText');
    if (typeof(buttonTextShowing) == 'undefined') {
        var broadcaster = document.getElementById('cmd_toggleButtonText');
        buttonTextShowing = broadcaster.hasAttribute('checked') &&
                            broadcaster.getAttribute('checked') == 'true';
        fromloading = true;
    }
    var tb1 = document.getElementById('standardToolbar');
    var tb2 = document.getElementById('debuggerToolbar');




    var toolbars = document.getElementsByTagName('toolbar');
    for (i = 0; i < toolbars.length; i++ ) {
        // Note: this can include custom toolbars
        _uilayout_setToolbarButtonText(toolbars[i].id, buttonTextShowing);
    }

    if (buttonTextShowing) {
        tb1.setAttribute('flex', '1')
        // When button labels are shown, the toolbars need to be in the following arrangement:
        var vbox = document.getElementById('toolbarsvbox');
        // if SCC
        var secondbox = document.getElementById('box-debugscc');
        secondbox.appendChild(tb2);



        vbox.appendChild(secondbox);
    } else {
        tb1.removeAttribute('flex');
        // When buttons are not shown, all the toolbars should be on the same line
        var hbox = document.getElementById('toolbarshbox');
        hbox.appendChild(tb1);
        hbox.appendChild(tb2);



    }
}


function uilayout_populatePreviewToolbarButton(popup)
{
    // Only do this once.
    // XXX We'll need to remove it's children when prefs are changed.
    if (popup.childNodes.length > 0)
        return;
    
    var koWebbrowser = Components.classes["@activestate.com/koWebbrowser;1"].
                       getService(Components.interfaces.koIWebbrowser);
    var browsers = koWebbrowser.get_possible_browsers(new Object());
    var mi;
    









    var browserURI;
    for (var i = 0; i < browsers.length; i++) {
        mi = document.createElementNS(XUL_NS, "menuitem");
        mi.setAttribute("label", browsers[i]);
        mi.setAttribute("crop", "center");
        mi.setAttribute("tooltiptext", uriparse_baseName(browsers[i]));
        browserURI = uriparse_localPathToURI(browsers[i]);
        mi.setAttribute("oncommand",
                        "gViewMgr.currentView.viewPreview('"+browserURI+"'); event.preventBubble();");
        popup.appendChild(mi);
    }
}

function uilayout_toggleTab(tabId)
{
    try {
        var tab = document.getElementById(tabId);
        // If the project/toolbox pane is not shown and focused, then show it
        // and focus on the relevant part manager
        var splitterWidget = document.getElementById('projectviewer-splitter');
        var partviewer;
        switch (tabId) {
            case 'project_tab':
                partviewer = document.getElementById('projectview');
                break;
            case 'toolbox_tab':
                partviewer = document.getElementById('toolboxview');
                break;
                
        }
        var tabs = document.getElementById('project_toolbox_tabs');
        if (splitterWidget.hasAttribute('collapsed') &&
            splitterWidget.getAttribute('collapsed') == 'true') {
            uilayout_toggleSplitter('cmd_viewProjects');
            tabs.selectedItem = tab;
            tabs.selectedItem.focus();
        } else {
            if (tabs.selectedItem != tab ||
                !partviewer.focused) {
                tabs.selectedItem = tab;
                tabs.selectedItem.focus();
            } else {
                // Before we collapse it, figure out whether the focus is in this
                // panel.  If so, then move it back to the editor
                if (isFocused(tabs.parentNode)) {
                    if (gViewMgr.currentView) {
                        gViewMgr.currentView.setFocus();
                    }
                }
                uilayout_toggleSplitter('cmd_viewProjects');
            }
        }
    } catch (e) {
        log.exception(e);
    }
}

function uilayout_updateViewMenu(menupopup)
{
    try {
        var viewOutput = document.getElementById('menu_viewOutput');
        viewOutput.setAttribute('checked', uilayout_outputPaneShown() ? 'true' : 'false');
    
        var viewProjectsToolbox = document.getElementById('menu_viewProjectsToolbox');
        viewProjectsToolbox.setAttribute('checked', uilayout_sidePaneShown() ? 'true' : 'false');
    } catch (e) {
        log.exception(e);
    }
    return true;
}

function uilayout_togglePane(splitterId, tabsId, cmdId)
{
    try {
        // If the project/toolbox pane is not shown, then show it
        // and focus on the relevant part manager
        var splitterWidget = document.getElementById(splitterId);
        var tabs = document.getElementById(tabsId);
        if (splitterWidget.hasAttribute('collapsed') &&
            splitterWidget.getAttribute('collapsed') == 'true') {
            uilayout_toggleSplitter(cmdId);
            tabs.selectedItem.focus();
        } else {
            // Before we collapse it, figure out whether the focus is in this
            // panel.  If so, then move it back to the editor
            if (isFocused(tabs.parentNode)) {
                if (gViewMgr.currentView) {
                    gViewMgr.currentView.setFocus();
                }
            }
            uilayout_toggleSplitter(cmdId);
        }
    } catch (e) {
        log.exception(e);
    }
}

function childOf(child, target) {
    // Returns true if 'child' is a descendant of 'target'
    if (!child || !target) return false;
    while (child) {
        if (child == target) {
            return true;
        }
        child = child.parentNode;
    }
    return false;
}

function isFocused(elt) {
    return childOf(top.document.commandDispatcher.focusedElement,
                   elt);
}


function uilayout_toggleProjects() {
    try {
        // If the project/toolbox pane is not shown, then show it
        // and focus on the relevant part manager
        var splitterWidget = document.getElementById("projectviewer-splitter");
        if (splitterWidget.hasAttribute('collapsed') &&
            splitterWidget.getAttribute('collapsed') == 'true') {
            uilayout_toggleSplitter('cmd_viewProjects');
            var tabs = document.getElementById('project_toolbox_tabs');
            if (tabs.selectedItem == document.getElementById('project_tab')) {
                document.getElementById('projectview').focus();
            } else {
                document.getElementById('toolboxview').focus();
            }
        } else {
            // Before we collapse it, figure out whether the focus is in this
            // panel.  If so, then move it back to the editor
            if (view_projectHasFocus() || view_toolboxHasFocus()) {
                if (gViewMgr.currentView) {
                    gViewMgr.currentView.setFocus();
                }
            }
            uilayout_toggleSplitter('cmd_viewProjects');
        }
    } catch (e) {
        log.exception(e);
    }
}

function uilayout_toggleSplitter(aCommandID) {
    var elt = document.getElementById(aCommandID);
    if (!elt) {
        _uilayout_log.error("couldn't find " + aCommandID);
        return;
    }
    var boxId = elt.getAttribute('box');
    var box = document.getElementById(boxId)
    if (!box) {
        _uilayout_log.error("couldn't find " + boxId);
        return;
    }
    var splitterId = elt.getAttribute('splitter')
    var splitter = document.getElementById(splitterId)
    if (!splitter) {
        _uilayout_log.error("couldn't find " + splitterId);
        return;
    }

    if (! box.hasAttribute('collapsed') || box.getAttribute("collapsed") == "false") {
        box.setAttribute('collapsed','true');
        splitter.setAttribute('collapsed','true');
    } else {
        box.setAttribute('collapsed','false');
        splitter.setAttribute('collapsed','false');
    }
}

function uilayout_FullScreen()
{
    window.fullScreen = !window.fullScreen;
    if (window.fullScreen) {
        window.maximize();
    } else {
        window.restore();
    }
}

function uilayout_onFullScreen()
{
  FullScreen.toggle();
}

function _uilayout_updateMRUMenu(prefName)
{
    // Update a MRU menu popup under the file menu.
    //    "prefName" indicate which MRU menu to update.
    //
    // XXX This code was significantly complitcated just for the special
    //     template MRU menu under File->New. Perhaps that should be
    //     factored out.
    var popupId, separatorId, prettyName;
    if (prefName == "mruProjectList") {
        popupId = "popup_mruProjects"; // MRU list is the whole popup.
        separatorId = null;
        prettyName = "Projects";
    } else if (prefName == "mruFileList") {
        popupId = "popup_mruFiles"; // MRU list is the whole popup.
        separatorId = null;
        prettyName = "Files";
    } else if (prefName == "mruTemplateList") {
        popupId = null;
        separatorId = "separator_mruTemplates"; // MRU list is everything after the separator.
        prettyName = "Templates";
    } else {
        throw("Unexpected MRU menu to update: prefName='"+prefName+"'");
    }

    var menupopup = popupId ? document.getElementById(popupId) : null;
    var separator = separatorId ? document.getElementById(separatorId) : null;
    var mruList = null;
    var menuitem;
    if (gPrefs.hasPref(prefName)) {
        mruList = gPrefs.getPref(prefName);
    }

    // Wipe out existing menuitems.
    if (separator) {
        menupopup = separator.parentNode;
        while (separator.nextSibling) {
            menupopup.removeChild(separator.nextSibling);
        }
    } else  {
        while (menupopup.firstChild) {
            menupopup.removeChild(menupopup.firstChild);
        }
    }

    if (mruList && mruList.length) {
        // Add a menuitem like the following for each entry in the MRU:
        //    <menuitem class="menuitem_mru"
        //              oncommand="open_openURI('URL');"
        //              label="URL_DISPLAY_NAME"/>
        // For template MRU entries use this instead:
        //    <menuitem class="menuitem_mru"
        //              oncommand="gViewMgr.doFileNewFromTemplate('URL');"
        //              label="URL_BASENAME"/>
        if (!menupopup) {
            menupopup = separator.parentNode;
        }
        var length = mruList.length;
        for (var i = 0; i < length; i++) {
            var url = mruList.getStringPref(i);
            menuitem = document.createElement("menuitem");
            // Mozilla does not handle duplicate accesskeys, so only putting
            // them on first 10.
            if ((i+1) <= 9) {
                menuitem.setAttribute("accesskey", "" + (i+1));
            } else if ((i+1) == 10) {
                menuitem.setAttribute("accesskey", "0");
            }
            if (prefName == "mruTemplateList") {
                menuitem.setAttribute("label", (i+1)+" "+uriparse_baseName(url));
            } else {
                menuitem.setAttribute("label", (i+1)+" "+uriparse_displayPath(url));
            }
            menuitem.setAttribute("class", "menuitem_mru");
            menuitem.setAttribute("crop", "center");
            // XXX:HACK: For whatever reason, the "observes" attribute is
            // ignored when the menu item is inside a popup, so we call
            // command_doCommand directly. THIS IS NOT A GOOD THING!
            if (prefName == "mruTemplateList") {
                menuitem.setAttribute("oncommand",
                                      "gViewMgr.doFileNewFromTemplate('" + url + "');");
            } else {
                menuitem.setAttribute("oncommand",
                                      "open_openURI('" + url + "')");
            }
            
            menupopup.appendChild(menuitem);
        }
    }
    
    // MRU is empty or does not exist
    else {
        // Add an empty one like this:
        //    <menuitem label="No Recent Files" disabled="true"/>
        menuitem = document.createElement("menuitem");
        menuitem.setAttribute("label", "No Recent "+prettyName);
        menuitem.setAttribute("disabled", true);
        menupopup.appendChild(menuitem);
    }
}

// Flags used to defer (re)building of the MRU menus until necessary.
var _gNeedToUpdateFileMRUMenu = false;
var _gNeedToUpdateProjectMRUMenu = false;
var _gNeedToUpdateTemplateMRUMenu = false;

function uilayout_UpdateMRUMenuIfNecessary(mru)
{
    // (Re)build the identified MRU menu if necessary.
    //    "mru" is either "project" or "file", indicating which MRU menu
    //        to update.
    if (mru == "project" && _gNeedToUpdateProjectMRUMenu) {
        _uilayout_updateMRUMenu("mruProjectList");
        _gNeedToUpdateProjectMRUMenu = false;
    } else if (mru == "file" && _gNeedToUpdateFileMRUMenu) {
        _uilayout_updateMRUMenu("mruFileList");
        _gNeedToUpdateFileMRUMenu = false;
    } else if (mru == "template" && _gNeedToUpdateTemplateMRUMenu) {
        _uilayout_updateMRUMenu("mruTemplateList");
        _gNeedToUpdateTemplateMRUMenu = false;
    }
}

var gUilayout_Observer = null;
function _uilayout_Observer ()
{
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                    getService(Components.interfaces.nsIObserverService);
    observerSvc.addObserver(this, "mru_changed",false);
    observerSvc.addObserver(this, "view_opened",false);
    observerSvc.addObserver(this, "view_closed",false);
    observerSvc.addObserver(this, "current_view_changed",false);
    observerSvc.addObserver(this, "current_view_language_changed",false);
};
_uilayout_Observer.prototype.destroy = function()
{
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                    getService(Components.interfaces.nsIObserverService);
    observerSvc.removeObserver(this, "mru_changed");
    observerSvc.removeObserver(this, "view_opened");
    observerSvc.removeObserver(this, "view_closed");
    observerSvc.removeObserver(this, "current_view_changed");
    observerSvc.removeObserver(this, "current_view_language_changed");
}
_uilayout_Observer.prototype.observe = function(subject, topic, data)
{
    _uilayout_log.info("Observing: " + topic);
    switch(topic) {
    case 'mru_changed':
        // Schedule update "File->Recent Files" and "File->Recent Projects"
        // menus.
        if (data == "mruFileList") {
            _gNeedToUpdateFileMRUMenu = true;
        } else if (data == "mruProjectList") {
            _gNeedToUpdateProjectMRUMenu = true;
        } else if (data == "mruTemplateList") {
            _gNeedToUpdateTemplateMRUMenu = true;
        }
        break;
    case 'current_view_changed':
        if (!gViewMgr.batchMode) {
            _uilayout_updateCurrentLanguage(subject);
            uilayout_updateTitlebar(subject);
        }
        // fall through
    case 'view_opened':
    case 'view_closed':
        _gNeedToUpdateWindowMenu = true;
        break;
    case 'current_view_language_changed':
        _uilayout_log.info("GOT current_view_language_changed");
        _uilayout_updateCurrentLanguage(subject);
        break;
    }
}

function _uilayout_updateCurrentLanguage(view)
{
    if (! _viewAsMenuIsBuilt) {
        // If we haven't built the menu yet, don't bother.
        return;
    }
    if (! view || !view.document || !view.document.language) {
        // If we don't have a current language, don't bother either
        return;
    }
    _uilayout_setCheckedLanguage(view.document.language);
}

function _uilayout_setCheckedLanguage(language)
{
    _uilayout_log.info("in _uilayout_updateCurrentLanguage");
    var languageNameNospaces = language.replace(' ', '', 'g');
    var id1 = "menu_viewAs" + languageNameNospaces
    var id2 = "contextmenu_viewAs" + languageNameNospaces
    var id;
    var childnodes = document.getElementById('popup_viewAsLanguage').childNodes;
    for (i = 0; i < childnodes.length; i++) {
        child = childnodes[i];
        id = child.getAttribute('id');
        if (id == id1) {
            child.setAttribute('checked', 'true');
        } else {
            child.setAttribute('checked', 'false');
        }
    }
    childnodes = document.getElementById('context-filetype-menu').childNodes;
    for (i = 0; i < childnodes.length; i++) {
        child = childnodes[i];
        id = child.getAttribute('id');
        if (id == id2) {
            child.setAttribute('checked', 'true');
        } else {
            child.setAttribute('checked', 'false');
        }
    }
}

var _gNeedToUpdateWindowMenu = false;
// Create and return on tab/window item at the bottom of the Window menu.
//
//  "view" is the view to which this menuitem is attached
//  "index" is the index in the list of views
//  "isCurrent" is a boolean indicating if the view is the current one.
//
function _uilayout_updateWindowList_createMenuItem(view, index, isCurrent)
{
    try {
        var menuitem = document.createElement('menuitem');
        menuitem.setAttribute('data', 'fileItem');
        menuitem.setAttribute('id', view.uid);
        menuitem.setAttribute("label", (index + 1) + " " + view.title);
        if (index+1 <= 9) {
            menuitem.setAttribute("accesskey", index+1);
        }
        menuitem.setAttribute('type', 'checkbox');
        if (isCurrent) {
            menuitem.setAttribute('checked', 'true');
            // No need to switch view oncommand, this view is already current.
            // However we *do* need to ensure that the checkmark stays.
            // See http://bugs.activestate.com/Komodo/show_bug.cgi?id=26423
            menuitem.setAttribute('oncommand',
                                  'event.target.setAttribute("checked", "true");');
        } else {
            menuitem.setAttribute('checked', 'false');
            menuitem.setAttribute('oncommand', 'this.view.makeCurrent();');
        }
        menuitem.view = view;
        return menuitem;
    } catch(ex) {
        _uilayout_log.exception(ex, "error generating Window list "+
                                    "menuitem for '"+view.title+"'");
    }
    return null;
}

// This updates the list in the Window menu.  The window menu calls
// this when it is being shown to reset itself.
function uilayout_updateWindowList(view, action, oldurl) {
    try {
        if (!_gNeedToUpdateWindowMenu) return;
        _gNeedToUpdateWindowMenu = false;
        
        var separator = document.getElementById('window-menu-separator');
        var popup = document.getElementById('popup_window');
        var views = gViewMgr.topView.getDocumentViews(true);
        // clear out checked items first
        var items = popup.getElementsByAttribute('data', 'fileItem');
        var i = 0;
        for (i=0; i < items.length; i++) {
            popup.removeChild(items[i]);
        }
        if (views.length == 0) {
            // re-enable this for first item
            separator.setAttribute('collapsed','true');
        } else {
            separator.removeAttribute('collapsed');
        }
        var mi;
        for (i=0; i < views.length; i++) {
            mi = _uilayout_updateWindowList_createMenuItem(views[i], i,
                    (views[i] == gViewMgr.currentView));
            if (mi) popup.appendChild(mi);
        }
    } catch(ex) {
        _uilayout_log.exception(ex, "error re-generating Window menu list");
    }
}

var _viewAsMenuIsBuilt = false;
function uilayout_UpdateViewAsMenuIfNecessary()
{
    if (_viewAsMenuIsBuilt) return;
    uilayout_buildViewAsLanguageMenu();
    _viewAsMenuIsBuilt = true;
}

// This updates the list in the View As ... menu.  
// Called by uilayout_onload
function uilayout_buildViewAsLanguageMenu() {
    gTimeline.enter('uilayout_buildViewAsLanguageMenu');
    // We may already have a language, let's find out:
    var language = null;
    if (gViewMgr.currentView &&
        gViewMgr.currentView.document &&
        gViewMgr.currentView.document.language) {
        language = gViewMgr.currentView.document.language;
    }
    var count = new Object();
    var temp = new Object();
    var languageRegistry = Components.classes["@activestate.com/koLanguageRegistryService;1"].
                           getService(Components.interfaces.koILanguageRegistryService);
    languageRegistry.getLanguages(temp,count);
    var languages = temp.value;
    var commandset = document.getElementById("cmdset_viewAs");
    languageRegistry.getAccessKeys(temp, count);
    var accesskeys = temp.value;
    var viewAsMenu = document.getElementById("popup_viewAsLanguage");
    var statusbarContextMenu = document.getElementById('context-filetype-menu');
    var menuitem, languageName, languageNameNospaces, cmd, i, menuitem2;
    for (i = 0; i < languages.length; i++) {
        languageName = languages[i];
        gTimeline.mark('handling language ' + languageName);
        languageNameNospaces = languageName.replace(' ', '', 'g')
        cmd = document.createElementNS(XUL_NS, 'command');
        cmd.setAttribute("id", "cmd_viewAs"+languageNameNospaces);
        if (language == null) {
            cmd.setAttribute("disabled", "true");
        }
        cmd.setAttribute("oncommand", "gViewMgr.do_ViewAs('" + languageName + "');");
        commandset.appendChild(cmd);
        menuitem = document.createElementNS(XUL_NS, 'menuitem');
        menuitem.setAttribute("id", "menu_viewAs" + languageNameNospaces);
        menuitem.setAttribute("accesskey", accesskeys[i]);
        menuitem.setAttribute("label", languageName);
        menuitem.setAttribute("type", "checkbox");
        menuitem.setAttribute("name", "current_language");
        menuitem.setAttribute("observes", "cmd_viewAs"+languageNameNospaces);
        menuitem2 = document.createElementNS(XUL_NS, 'menuitem');
        menuitem2.setAttribute("accesskey", accesskeys[i]);
        menuitem2.setAttribute("label", languageName);
        menuitem2.setAttribute("type", "checkbox");
        menuitem2.setAttribute("name", "current_language");
        menuitem2.setAttribute("observes", "cmd_viewAs"+languageNameNospaces);
        menuitem2.setAttribute("name", "current_language_statusbar");
        menuitem2.setAttribute("id", "contextmenu_viewAs" + languageNameNospaces);

        if (languageName == language) {
            menuitem.setAttribute('checked', 'true');
            menuitem2.setAttribute('checked', 'true');
        }

        viewAsMenu.appendChild(menuitem);
        statusbarContextMenu.appendChild(menuitem2);
    }
    cmd = document.createElementNS(XUL_NS, 'command');
    cmd.setAttribute("id", "cmd_viewAsGuessedLanguage");
    cmd.setAttribute("disabled", "true");
    cmd.setAttribute("oncommand", "gViewMgr.do_ViewAs('');");
    commandset.appendChild(cmd);
    menuitem = document.createElementNS(XUL_NS, 'menuseparator');
    viewAsMenu.appendChild(menuitem);
    menuitem2 = document.createElementNS(XUL_NS, 'menuseparator');
    statusbarContextMenu.appendChild(menuitem2);
    menuitem = document.createElementNS(XUL_NS, 'menuitem');
    menuitem.setAttribute("id", "menu_viewAsGuessedLanguage");
    menuitem.setAttribute("label", "Reset to best guess");
    menuitem.setAttribute("observes", "cmd_viewAsGuessedLanguage");
    viewAsMenu.appendChild(menuitem);
    menuitem2 = document.createElementNS(XUL_NS, 'menuitem');
    menuitem2.setAttribute("id", "menu_viewAsGuessedLanguage");
    menuitem2.setAttribute("label", "Reset to best guess");
    menuitem2.setAttribute("observes", "cmd_viewAsGuessedLanguage");
    statusbarContextMenu.appendChild(menuitem2);
    gTimeline.leave('uilayout_buildViewAsLanguageMenu');
}

function Komodo_SendCtrlKeyUpNotification()
{
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                      getService(Components.interfaces.nsIObserverService);
    try {
        observerSvc.notifyObservers(null, 'ctrl_keyup', null);
    } catch(e) {
        // Raises exception if no observers are registered. Just ignore that.
    }
}

function uilayout_outputPaneShown()
{
    var splitter = window.document.getElementById("redirect-splitter");
    if (!splitter.hasAttribute('collapsed')) {
        return true;
    }
    var collapsed = splitter.getAttribute('collapsed') == 'true';
    return !collapsed;
}

function uilayout_sidePaneShown()
{
    var splitter = window.document.getElementById("projectviewer-splitter");
    if (!splitter.hasAttribute('collapsed')) {
        return true;
    }
    var collapsed = splitter.getAttribute('collapsed') == 'true';
    return !collapsed;
}

//XXX When all uses of these three functions are just passing in 'window' as
//    the argument (as they soon will when the run dialog is re-factored) then
//    the arg should just be dropped.
function uilayout_ensureOutputPaneShown(mainWindow)
{
    var splitterWidget = mainWindow.document.getElementById("redirect-splitter");
    if (splitterWidget.hasAttribute('collapsed') && splitterWidget.getAttribute('collapsed') == 'true') {
        log.info('toggling splitter');
        uilayout_toggleSplitter('cmd_viewOutput');
    }
}

function uilayout_ensureProjectPaneShown(mainWindow)
{
    if (typeof(mainWindow) == 'undefined') mainWindow = window;
    var tabs = document.getElementById('project_toolbox_tabs');
    var tab = document.getElementById('project_tab');
    var splitterWidget = mainWindow.document.getElementById("projectviewer-splitter");
    if ((splitterWidget.hasAttribute('collapsed') && splitterWidget.getAttribute('collapsed') == 'true')
        || tabs.selectedItem != tab) {
        log.info('toggling splitter');
        uilayout_toggleTab('project_tab');
    }
}

function uilayout_ensureToolboxPaneShown(mainWindow)
{
    if (typeof(mainWindow) == 'undefined') mainWindow = window;
    var tabs = document.getElementById('project_toolbox_tabs');
    var tab = document.getElementById('toolbox_tab');
    var splitterWidget = mainWindow.document.getElementById("projectviewer-splitter");
    if ((splitterWidget.hasAttribute('collapsed') && splitterWidget.getAttribute('collapsed') == 'true')
        || tabs.selectedItem != tab) {
        log.info('toggling splitter');
        uilayout_toggleTab('toolbox_tab');
    }
}

/* Update the titlebar 
   Have to keep in mind debugging state */
function uilayout_updateTitlebar(view)  {
    var win = "ActiveState Komodo";
    var title = "";
    if (view != null)  {
        title = view.title;
        if (view.isDirty)  {
            title = title.concat("*");
        } else  {
            title = title.replace(/\*$/, "");
            title = title.replace(/(\s)+$/, "");              
        }
        if (view.document &&
            view.document.file &&
            view.getAttribute("type") != "startpage") {
            if (view.document.file.isLocal) {
                title = title + ' (' + view.document.file.dirName + ')';
            } else {
                title = view.document.displayPath;
            }
        }
        title = title + ' - ';
    } else {
        title="";      
    }      
    
    var debug = "";
    if (DBG.session.dbg.state == DBG.STATE_BREAK) {
        debug = " - Debugger is in Break Mode"
    } else if (DBG.session.dbg.state == DBG.STATE_RUNNING) {
        debug = " - Debugger is Running";
    }

    window.title = (title + win + debug);
}


function uilayout_unload()
{
    gUilayout_Observer.destroy();
    gUilayout_Observer = null;
}

function uilayout_onload()
{
    gTimeline.enter("uilayout_onload");
    uilayout_updateToolbarArrangement();
    addEventListener("fullscreen", uilayout_onFullScreen, false);
    _gNeedToUpdateFileMRUMenu = true;
    _gNeedToUpdateProjectMRUMenu = true;
    _gNeedToUpdateTemplateMRUMenu = true;
    gUilayout_Observer = new _uilayout_Observer();
    KomodoRegisterOnUnloadHandler(uilayout_unload);
    gTimeline.leave("uilayout_onload");
}
