/* Copyright (c) 2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/* Left to do in order of priority
  - Dealing with encodings
  - Test on linux
*/

var enumerator = Components.classes["@mozilla.org/gfx/fontenumerator;1"].createInstance();
if( enumerator )
    enumerator = enumerator.QueryInterface(Components.interfaces.nsIFontEnumerator);

const tag = 'chrome://jaguar/content/pref/pref-alllangfonts.xul';
var dialog;
var log;
var inFontsAndColors;
var gFontTypes   = ["serif","sans-serif", /*"cursive", "fantasy",*/"monospace"];
var gFontLanguages = ['x-western','x-central-euro','ja','zh-TW',
                      'zh-CN','ko','x-cyrillic','x-baltic','el',
                      'tr','x-unicode','th','he','ar'];

var gLanguageRegistry = Components.classes["@activestate.com/koLanguageRegistryService;1"]
            .getService(Components.interfaces.koILanguageRegistryService);


function loadSample()
{
    language = dialog.currentLanguage;
    var languageSvc = Components.classes[
            "@activestate.com/koLanguage?language=" + language + ";1"].
            getService(Components.interfaces.koILanguage);
    sample = languageSvc.sample;
    if (! sample) {
        sample = "No sample for " + language + " available in this interim release.";
    }
    dialog.scintilla.initWithBuffer(sample, language);
    dialog.scintilla.anchor = sample.length/4;
    dialog.scintilla.currentPos = sample.length/2;
}

function PrefLangFonts_OnLoad()  {
    try {
        trace_onload();
        command_onload();
        log = getLoggingMgr().getLogger('pref-allfonts');
        dialog = initDialog();
        onSampleBlur();
        parent.hPrefWindow.onpageload(tag);
        parent.hPrefWindow.registerOKCallbackFunc(PrefFonts_OkCallback, tag);
        parent.hPrefWindow.registerCancelCallbackFunc(PrefFonts_CancelCallback, tag);
        dialog.scintilla.onPosChangedCB = onPosChanged;
    } catch (e) {
        log.error(e);
    }
}

function onPosChanged(position)
{
    try {
        var styleno = dialog.scintilla.scimoz.getStyleAt(position);
        var common = dialog.currentScheme.getCommonName(dialog.currentLanguage,
                                                        styleno);
        if (common) {
            dialog.commonlist.value = common;
            updateCommonStyle();
        }
        var specific = dialog.currentScheme.getSpecificName(dialog.currentLanguage,
                                                            styleno);
        if (specific) {
            dialog.specificlist.value = specific;
            updateSpecificStyle();
        }
    } catch (e) {
        log.error(e);
    }
}

// This function builds the 'scheme' menulist from the schemeservice
function setupSchemes()
{
    try {
        var schemes = new Array();
        dialog.schemeService.getSchemeNames(schemes, new Object());
        var menuitem;
        var s
        for (var i = 0; i < schemes.value.length; i++) {
            scheme = schemes.value[i];
            menuitem = document.createElement('menuitem');
            s = dialog.schemeService.getScheme(scheme);
            if (! s.writeable) {
                menuitem.setAttribute('class','primary_menu_item');
            }
            menuitem.setAttribute('label', scheme);
            menuitem.setAttribute('id', scheme);
            menuitem.setAttribute('value', scheme);
            dialog.schemespopup.appendChild(menuitem);
        }
        dialog.schemeslist.value = dialog.currentScheme.name;
    } catch (e) {
        log.error(e);
    }
}

function updateDelete()
{
    if (dialog.currentScheme.writeable) {
        if (dialog.deleteButton.hasAttribute('disabled')) {
            dialog.deleteButton.removeAttribute('disabled');
        }
    } else {
        dialog.deleteButton.setAttribute('disabled', 'true');
    }
}

function updateFromScheme()
{
    try {
        if (dialog.currentScheme.preferFixed) {
            dialog.preferFixed.setAttribute('selected', 'true');
            dialog.preferProp.setAttribute('selected', 'false');
        } else {
            dialog.preferFixed.setAttribute('selected', 'false');
            dialog.preferProp.setAttribute('selected', 'true');
        }
        dialog.useSelFore.setAttribute('checked',
                                        dialog.currentScheme.useSelFore);
        updateMenuitemAndCheckbox('useSelFore', 'selFore');
        dialog.caretLineVisible.setAttribute('checked',
                                             dialog.currentScheme.caretLineVisible);
        updateMenuitemAndCheckbox('caretLineVisible', 'caretLineBack');
        updateScintilla();
        updateSchemeColor(dialog.extracolors.selectedItem.getAttribute('id'));
        updateFonts();
        updateDelete();
        updateCommonPopup();
        updateSpecificPopup();
    } catch (e) {
        log.error(e);
    }
}

function updateScintilla()
{
    var scheme = dialog.currentScheme;
    var scintilla = dialog.scintilla.scimoz;
    var encoding = dialog.currentEncoding;
    var alternateType = false;
    scheme.applyScheme(scintilla, dialog.currentLanguage, encoding, alternateType);
}

function preferFixed(truefalse)
{
    if (!ensureWriteableScheme()) return;
    dialog.currentScheme.preferFixed = truefalse;
    updateScintilla();
}

function setColour(colorpicker)
{
    var colorid;
    var color = colorpicker.color;
    var colorpickerid = colorpicker.getAttribute('id')
    if (!ensureWriteableScheme()) {
        return;
    }
    if (colorpickerid == 'schemeColorPicker') {
        colorid = dialog.extracolors.selectedItem.getAttribute('id')
        dialog.currentScheme.setColor(colorid, color);
        updateScintilla();
    } else {
        colorid = colorpickerid;
        switch (colorid) {
            case 'fixedColorPickerFore':
                dialog.currentScheme.setFore('', faceIdentifier(dialog.currentEncoding, 1), color);
                break;
            case 'fixedColorPickerBack':
                dialog.currentScheme.setBack('', faceIdentifier(dialog.currentEncoding, 1), color);
                break;
            case 'propColorPickerFore':
                dialog.currentScheme.setFore('', faceIdentifier(dialog.currentEncoding, 0), color);
                break;
            case 'propColorPickerBack':
                dialog.currentScheme.setBack('', faceIdentifier(dialog.currentEncoding, 0), color);
                break;
            case 'commonColorPickerFore':
                dialog.currentScheme.setFore('',
                                             dialog.currentCommonStyle,
                                             color);
                break;
            case 'commonColorPickerBack':
                dialog.currentScheme.setBack('',
                                             dialog.currentCommonStyle,
                                             color);
                break;
            case 'specificColorPickerFore':
                dialog.currentScheme.setFore(dialog.currentLanguage,
                                             dialog.currentSpecificStyle,
                                             color);
                break;
            case 'specificColorPickerBack':
                dialog.currentScheme.setBack(dialog.currentLanguage,
                                             dialog.currentSpecificStyle,
                                             color);
                break;
            default:
                log.error("setColour called with unknown colorid: " + colorid);
        }
        updateScintilla();
        updateCommonStyle();
    }
}

function onClickBoldOrItalic(button)
{
    try {
        if (!ensureWriteableScheme()) {
            // undo the change
            if (button.hasAttribute('checked')) {
                button.removeAttribute('checked');
            } else {
                button.setAttribute('checked', 'true');
            }
            return;
        }
        var updateCommon = false;
        var updateSpecific = false;
        var id = button.getAttribute('id');
        var checked = button.hasAttribute('checked') ? false : true;
        switch (id) {
            case 'fixedBold':
                dialog.currentScheme.setBold('',
                                             faceIdentifier(dialog.currentEncoding, 1),
                                             checked);
                updateCommon = true;
                break;
            case 'fixedItalic':
                dialog.currentScheme.setItalic('',
                                               faceIdentifier(dialog.currentEncoding, 1),
                                               checked);
                updateCommon = true;
                break;
            case 'propBold':
                dialog.currentScheme.setBold('',
                                             faceIdentifier(dialog.currentEncoding, 0),
                                             checked);
                updateCommon = true;
                break;
            case 'propItalic':
                dialog.currentScheme.setItalic('',
                                               faceIdentifier(dialog.currentEncoding, 0),
                                               checked);
                updateCommon = true;
                break;
            case 'commonItalic':
                dialog.currentScheme.setItalic('',
                                               dialog.currentCommonStyle,
                                               checked);
                updateSpecific = true;
                break;
            case 'commonBold':
                dialog.currentScheme.setBold('',
                                             dialog.currentCommonStyle,
                                             checked);
                updateSpecific = true;
                break;
            case 'specificItalic':
                dialog.currentScheme.setItalic(dialog.currentLanguage,
                                               dialog.currentSpecificStyle,
                                               checked);
                break;
            case 'specificBold':
                dialog.currentScheme.setBold(dialog.currentLanguage,
                                             dialog.currentSpecificStyle,
                                             checked);
                break;
        }
        updateScintilla();
        if (updateCommon) {
            updateCommonStyle();
        } else if (updateSpecific) {
            updateSpecificStyle();
        }
    } catch (e) {
        log.error(e);
    }
}

function onClickFixedFont()
{
    try {
        if (!ensureWriteableScheme()) return;
        var face = dialog.fixedList.value;
        dialog.currentScheme.setFont(faceIdentifier(dialog.currentEncoding, 1), face);
        updateScintilla();
    } catch (e) {
        log.error(e);
    }
}

function onClickFixedSize()
{
    try {
        if (!ensureWriteableScheme()) return;
        var size = dialog.fixedSize.value;
        dialog.currentScheme.setSize('', faceIdentifier(dialog.currentEncoding, 1), size);
        updateScintilla();
    } catch (e) {
        log.error(e);
    }
}

function onClickProportionalFont()
{
    try {
        if (!ensureWriteableScheme()) return;
        var face = dialog.propList.value;
        dialog.currentScheme.setFont(faceIdentifier(dialog.currentEncoding, 0), face);
        updateScintilla();
    } catch (e) {
        log.error(e);
    }
}

function onClickProportionalSize()
{
    try {
        if (!ensureWriteableScheme()) return;
        var size = dialog.propSize.value;
        dialog.currentScheme.setSize('', faceIdentifier(dialog.currentEncoding, 0), size);
        updateScintilla();
    } catch (e) {
        log.error(e);
    }
}

function onClickCommonSize()
{
    try {
        if (!ensureWriteableScheme()) return;
        var size = dialog.commonSize.value;
        dialog.currentScheme.setSize('',
                                     dialog.currentCommonStyle,
                                     size);
        updateScintilla();
        updateSpecificStyle();
    } catch (e) {
        log.error(e);
    }
}

function onClickSpecificSize()
{
    try {
        if (!ensureWriteableScheme()) return;
        var size = dialog.specificSize.value;
        dialog.currentScheme.setSize(dialog.currentLanguage,
                                     dialog.currentSpecificStyle,
                                     size);
        updateScintilla();
    } catch (e) {
        log.error(e);
    }
}
function schemePick(event) {
    try {
        // If the current menuitem is the 'blank' one, then delete it
        // -- the scheme will get deleted automatically.
        dialog.currentScheme = dialog.schemeService.getScheme(dialog.schemeslist.value);
        updateFromScheme();
    } catch (e) {
        log.error(e);
    }
}

function doDelete()
{
    try {
        var name = dialog.currentScheme.name 
        if (dialog_yesNo("Are you sure you want to delete the scheme '" + name +"'?  This action cannot be undone.") == 'No') {
            return;
        }
        dialog.currentScheme.remove();
        dialog.currentScheme = dialog.schemeService.getScheme('Default');
        var oldScheme = dialog.prefset.getStringPref('editor-scheme');
        if (oldScheme == name) {
            // we _must_ change the pref, since that scheme is gone even before the OK.
            dialog.prefset.setStringPref('editor-scheme', 'Default');
            // We must do it on the parent's opener because of the pref dialog's hadnling of prefs--
            // our prefset is ignored on Cancel.
            parent.opener.gPrefs.setStringPref('editor-scheme', 'Default');
        }
        // need to remove it from the popup
        var menuitem = document.getElementById(name);
        menuitem.parentNode.removeChild(menuitem);
        dialog.schemeslist.value = dialog.currentScheme.name
        updateFromScheme();
    } catch (e) {
        log.exception(e);
    }
}

function doNew()
{
    try {
        if (dialog.currentScheme.unsaved) {
            var answer = dialog_yesNoCancel("Save current scheme before creating new one?");
            if (answer == "Cancel") {
                return false;
            }
            if (answer == "Yes") {
                dialog.currentScheme.save()
            }
        }
        var newSchemeName = '';
        var schemes = new Array();
        dialog.schemeService.getSchemeNames(schemes, new Object());
        schemes = schemes.value;
        var i, ok;
        var fnameRe = /^[\w\d ]*$/;
        while (1) {
            newSchemeName = dialog_prompt("Enter a new scheme name.  The new scheme will be based on the currently selected '" +
                                          dialog.currentScheme.name + "' scheme.", "New Scheme Name:",
                                          newSchemeName // default value
                                          );
            if (!newSchemeName) {
                return false;
            }
            ok = true;
            // Check to make sure that the name isn't already taken and that it can be written to disk.
            for (i = 0; i < schemes.length; i++) {
                if (schemes[i] == newSchemeName) {
                    alert("The scheme name '" + newSchemeName + "' is already used.  Please choose another.")
                    ok = false;
                    break;
                }
            }
            if (!ok) continue;
            if (fnameRe.test(newSchemeName)) {
                ok = true;
                break;
            }
            alert("Scheme names must contain alphanumeric characters, spaces or underscores only.  Please choose another name.");
        }
    
        var newScheme = dialog.currentScheme.clone(newSchemeName);
        var menuitem = document.createElement('menuitem');
        menuitem.setAttribute('label', newSchemeName);
        menuitem.setAttribute('id', newSchemeName);
        menuitem.setAttribute('value', newSchemeName);
        dialog.schemespopup.appendChild(menuitem);
        dialog.schemeslist.selectedItem = menuitem;
        dialog.currentScheme = newScheme;
        updateScintilla();
        return true;
    } catch (e) {
        log.exception(e);
    }
}

function ensureWriteableScheme()
{
    if (dialog.currentScheme.writeable) {
        // Nothing to do, the current scheme can handle it.
        dialog.modified = true;
        return true;
    }
    if (!doNew()) {
        updateFromScheme();
        return false;
    } else {
        dialog.modified = true;
        return true;
    }
}

function pickFaceType(menulist)
{
    try {
        if (!ensureWriteableScheme()) return;
        var id = menulist.getAttribute('id');
        var useFixed = menulist.selectedItem.getAttribute('value') == '1';
        if (id == 'fixedOrPropCommon') {
            dialog.currentScheme.setFaceType('',
                                             dialog.currentCommonStyle,
                                             useFixed);
        } else {
            dialog.currentScheme.setFaceType(dialog.currentLanguage,
                                             dialog.currentSpecificStyle,
                                             useFixed);
        }
        updateScintilla();
        updateCommonStyle();
    } catch (e) {
        log.error(e);
    }
}

function OnPreferencePageInitalize(prefset) {
    try {
        var data = new Object();
        data.prefset = prefset;
        data.currentLanguage = 'Python'; // get from current view
        var schemeName = data.prefset.getStringPref('editor-scheme');
        data.schemeService = Components.classes['@activestate.com/koScintillaSchemeService;1'].getService()
        data.currentScheme = data.schemeService.getScheme(schemeName);
    } catch (e) {
        log.error(e);
    }
    return data;
}

function OnPreferencePageUnloading(prefset, data) {
    try {
        data.currentScheme = dialog.currentScheme
        data.modified = dialog.modified;
        inFontsAndColors = null;
    } catch (e) {
        log.exception(e);
    }
}

function OnPreferencePageLoading(prefset, data) {
    try {
        data.prefset = prefset;
        generateFontList();
        dialog.prefset = prefset;
        dialog.schemeService = data.schemeService;
        dialog.currentScheme = data.currentScheme;
        dialog.currentLanguage = data.currentLanguage;
        dialog.scintilla.scimoz.setMarginWidthN(2,0);
        dialog.scintilla.scimoz.setMarginWidthN(1,0);
        dialog.scintilla.scimoz.setMarginWidthN(0,0);
        dialog.languageList.selection = dialog.currentLanguage;
        dialog.currentEncoding = 'default';
        updateEncodingPopup();
        setupSchemes();
        updateLanguage();
        updateFromScheme();
        initFonts();
        inFontsAndColors = data;
        updateEncodingReset();
    } catch (e) {
        log.error(e);
    }
}

function onResetEncoding(event)
{
    try {
        if (!ensureWriteableScheme()) return;
        dialog.currentScheme.resetStyle('',
                                        faceIdentifier(dialog.currentEncoding,
                                                       dialog.currentScheme.preferFixed));
        updateScintilla();
        updateFromScheme();
    } catch (e) {
        log.error(e);
    }
}

function onResetCommon(event)
{
    try {
        if (!ensureWriteableScheme()) return;
        dialog.currentScheme.resetStyle('',
                                        dialog.currentCommonStyle);
        updateScintilla();
        updateCommonStyle();
    } catch (e) {
        log.error(e);
    }
}

function onResetSpecific(event)
{
    try {
        if (!ensureWriteableScheme()) return;
        dialog.currentScheme.resetStyle(dialog.currentLanguage,
                                        dialog.currentSpecificStyle);
        updateScintilla();
        updateSpecificStyle();
    } catch (e) {
        log.error(e);
    }
}

function initDialog() {
    // Get a handle on the UI widgets
    dialog = {};
    dialog.modified = false;
    dialog.scintilla = document.getElementById('sample');
    dialog.deleteButton = document.getElementById('deleteScheme');
    dialog.encodingReset = document.getElementById('encodingReset');
    dialog.schemeslist = document.getElementById('schemeslist');
    dialog.schemespopup = document.getElementById('schemespopup');
    dialog.fixedList = document.getElementById('fixed');
    dialog.propList = document.getElementById('proportional');
    dialog.fixedSize = document.getElementById('fixedSize');
    dialog.propSize = document.getElementById('propSize');
    dialog.fixedBold = document.getElementById('fixedBold');
    dialog.fixedItalic = document.getElementById('fixedItalic');
    dialog.propBold = document.getElementById('propBold');
    dialog.propItalic = document.getElementById('propItalic');
    dialog.preferFixed = document.getElementById('preferFixed');
    dialog.preferProp = document.getElementById('preferProp');
    dialog.fixedColorPickerFore = document.getElementById('fixedColorPickerFore');
    dialog.fixedColorPickerBack = document.getElementById('fixedColorPickerBack');
    dialog.propColorPickerFore = document.getElementById('propColorPickerFore');
    dialog.propColorPickerBack = document.getElementById('propColorPickerBack');
    // second tab: colors
    dialog.extracolors = document.getElementById('extracolors');
    dialog.schemeColorPicker = document.getElementById('schemeColorPicker');
    dialog.useSelFore = document.getElementById('useSelFore');
    dialog.caretLineVisible = document.getElementById('caretLineVisible');
    // third tab: common
    dialog.commonlist = document.getElementById('commonlist')
    dialog.commonFore = document.getElementById('commonColorPickerFore')
    dialog.commonBack = document.getElementById('commonColorPickerBack')
    dialog.commonSize = document.getElementById('commonSize');
    dialog.commonFaceType = document.getElementById('fixedOrPropCommon');
    dialog.commonBold = document.getElementById('commonBold')
    dialog.commonItalic = document.getElementById('commonItalic')
    // last tab: specific
    dialog.specificlist = document.getElementById('specificlist')
    dialog.specificFore = document.getElementById('specificColorPickerFore')
    dialog.specificBack = document.getElementById('specificColorPickerBack')
    dialog.specificBold = document.getElementById('specificBold')
    dialog.specificItalic = document.getElementById('specificItalic')
    dialog.languageList = document.getElementById('languageList');
    dialog.encodingslist = document.getElementById('encodingslist');
    dialog.specificSize = document.getElementById('specificSize');
    dialog.specificFaceType = document.getElementById('fixedOrPropSpecific');
    return dialog;
}

function toggleMenuitem(checked, menuid)
{
    menuitem = document.getElementById(menuid);
    if (checked) {
        if (menuitem.hasAttribute('disabled')) {
            menuitem.removeAttribute('disabled');
        }
    } else {
        menuitem.setAttribute('disabled', 'true');
        if (dialog.extracolors.selectedItem == menuitem) {
            dialog.extracolors.selectedItem = document.getElementById('caretFore');
            updateSchemeColor('caretFore')
        }
    }
}

function updateMenuitemAndCheckbox(checkboxid, menuid)
{
    try {
        var checkbox = document.getElementById(checkboxid);
        var checked = checkbox.getAttribute('checked') == "true";
        switch (checkboxid) {
            case 'useSelFore':
                dialog.currentScheme.useSelFore = checked;
                toggleMenuitem(checked, menuid);
                break;
            case 'caretLineVisible':
                dialog.currentScheme.caretLineVisible = checked;
                toggleMenuitem(checked, menuid);
                break;
            default:
                log.exception("got updateMenuitemAndCheckbox with unknown checkboxid: "+ checkboxid);
        }
    } catch (e) {
        log.error(e);
    }
}

function clickCheckbox(id, menuid)
{
    try {
        if (!ensureWriteableScheme()) return;
        updateMenuitemAndCheckbox(id, menuid);
        updateScintilla();
    } catch (e) {
        log.error(e);
    }
}

function updateEncodingPopup()
{
    var koEncodingServices = Components.classes["@activestate.com/koEncodingServices;1"]
        .getService(Components.interfaces.koIEncodingServices);

    var temp = new Object();
    koEncodingServices.enumerateEncodings(temp,new Object());
    gEncodings = temp.value;
    
    var popup = document.getElementById('encodingspopup');
    var item = document.createElement('menuitem');
    item.setAttribute('value','default');
    item.setAttribute('label','Default');
    popup.appendChild(item);
    for (var i=0; i< gEncodings.length; i++) {
        item = document.createElement('menuitem');
        item.setAttribute('value',gEncodings[i].python_encoding_name);
        item.setAttribute('label',gEncodings[i].friendly_encoding_name);
        popup.appendChild(item);
    }
    document.getElementById('encodingslist').value = 'default';
}

function updateCommonPopup()
{
    try{
        labels = new Array();
        dialog.schemeService.getCommonStyles(labels, new Object());
        labels = labels.value;
        var popup = document.getElementById('commonpopup');
        for (var i=0; i< labels.length; i++) {
            item = document.createElement('menuitem');
            item.setAttribute('value',labels[i]);
            item.setAttribute('label',labels[i]);
            popup.appendChild(item);
        }
        dialog.commonlist.value = labels[0];
        updateCommonStyle();
    } catch (e) {
        log.error(e);
    }
}

function updateSpecificPopup()
{
    try{
        // Grab a list of available styles.
        labels = new Array();
        dialog.schemeService.getLanguageStyles(dialog.currentLanguage, labels, new Object());
        labels = labels.value;
        var popup = document.getElementById('specificpopup');
        // Remove whatever is in the menu
        var children = popup.childNodes;
        var i, child
        // We start by cleaning out old menu items (keeping the 'keep' marked ones)
        for (i = children.length-1; i >= 0; i--) {
            child = children[i];
            popup.removeChild(child);
        }
        // Now we add what this language needs.
        for (i = 0; i < labels.length; i++) {
            item = document.createElement('menuitem');
            item.setAttribute('value',labels[i]);
            item.setAttribute('label',labels[i]);
            popup.appendChild(item);
        }
        document.getElementById('specificlist').value = labels[0];
        updateSpecificStyle();
    } catch (e) {
        log.error(e);
    }
}

function updateCommonStyle()
{
    try {
        var style = dialog.currentCommonStyle = document.getElementById('commonlist').value;
        if (!style) {
            return;
        }
        var fore, back, bold, italic, facetype, size;
        fore = dialog.currentScheme.getFore('', style);
        dialog.commonFore.color = fore;
        back = dialog.currentScheme.getBack('', style);
        dialog.commonBack.color = back;
        bold = dialog.currentScheme.getBold('', style);
        setCheckboxButton(dialog.commonBold, bold);
        italic = dialog.currentScheme.getItalic('', style);
        setCheckboxButton(dialog.commonItalic, italic);
        size = dialog.currentScheme.getSize('', style);
        dialog.commonSize.value = size;
        useFixed = dialog.currentScheme.getFaceType('', style);
        //dump("got useFixed = " + useFixed + '\n');
        dialog.commonFaceType.selectedIndex = 1-useFixed;
        //gTimeline.dumpDOM(dialog.commonFaceType);
        // Whenever the common style may have changed, the specific style may have changed
        updateSpecificStyle();
    } catch (e) {
        log.error(e);
    }
}

function setCheckboxButton(element, checked)
{
    //gTimeline.dumpDOM(element);
    if (checked) {
        //element.setAttribute('checkState', '1');
        element.setAttribute('checked', 'true');
    } else {
        //element.setAttribute('checkState', '0');
        if (element.hasAttribute('checked')) {
            element.removeAttribute('checked');
        }
    }
}

function updateSpecificStyle()
{
    var style = dialog.currentSpecificStyle = document.getElementById('specificlist').value;
    if (!style) return;
    var fore, back, bold, italic, facetype, size;
    fore = dialog.currentScheme.getFore(dialog.currentLanguage, style);
    dialog.specificFore.color = fore;
    back = dialog.currentScheme.getBack(dialog.currentLanguage, style);
    dialog.specificBack.color = back;
    bold = dialog.currentScheme.getBold(dialog.currentLanguage, style);
    setCheckboxButton(dialog.specificBold, bold);
    italic = dialog.currentScheme.getItalic(dialog.currentLanguage, style);
    setCheckboxButton(dialog.specificItalic, italic);
    size = dialog.currentScheme.getSize(dialog.currentLanguage, style);
    dialog.specificSize.value = size;
    facetype = dialog.currentScheme.getFaceType(dialog.currentLanguage, style);
    dialog.specificFaceType.selectedIndex = 1-facetype;
}

function updateSchemeColor(colorid) {
    try {
        var colorvalue = dialog.currentScheme.getColor(colorid);
        dialog.schemeColorPicker.color = colorvalue;
    } catch (e) {
        log.error(e);
    }
}

function PrefFonts_OkCallback(prefset, data)  {
    try {
        var schemeName;
        if (typeof(inFontsAndColors) != 'undefined') {
            source = dialog;
        } else {
            source = data;
        }
        schemeName = source.currentScheme.name;
        // If we are dealing with a new scheme or a scheme we've changed, then save it.
        if (source.currentScheme.unsaved || source.currentScheme.isDirty) {
            source.currentScheme.save()
        }
        oldScheme = data.prefset.getStringPref('editor-scheme');
        if (oldScheme != schemeName) {
            source.prefset.setStringPref('editor-scheme', schemeName);
        } else if (source.modified) {
            observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                            getService(Components.interfaces.nsIObserverService);
            observerSvc.notifyObservers(this, 'scheme-changed', schemeName);
        }
    } catch (e) {
        log.error(e);
    }
    return true;
}

function PrefFonts_CancelCallback(prefset, data)  {
    try {
        if (typeof(inFontsAndColors) != 'undefined') {
            source = dialog;
        } else {
            source = data;
        }
        schemeName = source.currentScheme.name;
        // If we've modified an existing scheme, revert it
        if (!source.currentScheme.unsaved && source.currentScheme.isDirty) {
            source.currentScheme.revert()
        }
        schemeService = Components.classes['@activestate.com/koScintillaSchemeService;1'].getService()
        schemeService.purgeUnsavedSchemes();
        return true;
    } catch (e) {
        log.exception(e);
    }
}

var gFontsProp;
var gFontsFixed;
var gFontsAll;

function generateFontList()
{
try {
    // get all the fonts on the system
    var strFontSpecs;
    var j;
    gFontsProp = [];
    gFontsFixed = [];
    gFontsAll = [];
    gFontNames = {};
    var re = /^([^-]+)-([^-]+)/;
    for (var i=0;i<gFontLanguages.length;i++) {

        strFontSpecs = enumerator.EnumerateFonts(gFontLanguages[i], 
                                                 'sans',
                                                 new Object());
        var fontspec;
        for (j=0; j < strFontSpecs.length; j++) {
            if (typeof(gFontNames[strFontSpecs[j]])=='undefined' ||
                !gFontNames[strFontSpecs[j]]) {
                var n = re.exec(strFontSpecs[j]);
                if (!n) continue; // Skip over font names we can't handle
                var pn = n[2]+' ('+n[1]+')';
                gFontNames[strFontSpecs[j]] = pn;
		// Mozilla doesn't tell us which fonts are fixed
                // width and which are proportional =(
                // We could guess based on the names?
                gFontsFixed.push(strFontSpecs[j]);
                gFontsProp.push(strFontSpecs[j]);
            }
        }





















    }
} catch (e) {
    log.exception(e);
}
}

function initFonts(encoding)  {
    if (typeof(encoding)=='undefined') {
       encoding = parent.opener.gPrefs.getStringPref('encodingDefault');
    }
    // Do all fonts list first
    var fixedElement = new listElement( 'fixed' );
    var propElement = new listElement( 'proportional' );
    var strFontFaces = [];
    var strFontSpecs = {};
    // Fixed and Proportional
    fixedElement.clearList();
    propElement.clearList();


    // We need to convert from encoding (e.g. utf-8) to fontspec for
    // that encoding (e.g. iso10646-1)
    var koEncodingServices = Components.classes["@activestate.com/koEncodingServices;1"]
        .getService(Components.interfaces.koIEncodingServices);
    var fontspec = koEncodingServices.get_encoding_info(encoding).fontspec;
    var re = /^([^-]+)-([^-]+)-(.*)/;
    for (var j=0; j < gFontsProp.length; j++) {
        var n = re.exec(gFontsProp[j]);
        if (n && fontspec == n[3]) {
            strFontFaces[strFontFaces.length] = gFontNames[gFontsProp[j]];
            strFontSpecs[gFontNames[gFontsProp[j]]] = gFontsProp[j];
        }
    }
    propElement.appendStrings(strFontFaces,strFontSpecs);
    fixedElement.appendStrings(strFontFaces,strFontSpecs);




}

function updateEncodingReset()
{
    try {
        // If the currently selected encoding is not "Default", enable the "Reset" button
        if (dialog.currentEncoding == 'default') {
            dialog.encodingReset.setAttribute('disabled', 'true');
        } else {
            if (dialog.encodingReset.hasAttribute('disabled')) {
                dialog.encodingReset.removeAttribute('disabled');
            }
        }
    } catch (e) {
        log.exception(e);
    }
}

function selectEncoding() {
    try {
        dialog.currentEncoding = dialog.encodingslist.value;
        initFonts(dialog.currentEncoding);
        dialog.scintilla.scintilla.encoding = dialog.currentEncoding;
        updateFromScheme();
        updateScintilla();
        updateEncodingReset();
    } catch (e) {
        log.exception(e);
    }
}

function _findFontName(name) {
    if (typeof(gFontNames[name]) == 'undefined') {
        return name;
    }
    return gFontNames[name];
}

function customColor(colorpickerid) {
    colorpicker = document.getElementById(colorpickerid);
    var sysUtils = Components.classes['@activestate.com/koSysUtils;1'].
                    getService(Components.interfaces.koISysUtils);
    var color = colorpicker.color;
    newcolor = sysUtils.pickColor(color);
    if (newcolor) {
        if (!ensureWriteableScheme()) return;
        colorpicker.color = newcolor;
        setColour(colorpicker);
    }
}

function updateLanguage()
{
    try {
        loadSample();
        updateSpecificPopup();
    } catch (e) {
        log.error(e);
    }
}

function changeLanguage() {
    dialog.currentLanguage = dialog.languageList.selection;
    updateLanguage();
    updateScintilla();
}

function updateFonts()  {
    // order is font, size, back, fore, italic, bold
    
    var fLabel = dialog.currentScheme.getFont(faceIdentifier(dialog.currentEncoding, 1));
    var pLabel = dialog.currentScheme.getFont(faceIdentifier(dialog.currentEncoding, 0));
    var fSize = dialog.currentScheme.getSize('', faceIdentifier(dialog.currentEncoding, 1));
    var pSize = dialog.currentScheme.getSize('', faceIdentifier(dialog.currentEncoding, 0));
    var m;

    dialog.fixedList.setAttribute('label', _findFontName(fLabel));
    dialog.propList.setAttribute('label', _findFontName(pLabel));
    dialog.fixedSize.setAttribute('label', fSize);
    m = document.getElementById('fixedSize_'+fSize);
    dialog.fixedSize.selectedItem = m;
    dialog.propSize.setAttribute('label', pSize);
    m = document.getElementById('propSize_'+pSize);
    dialog.propSize.selectedItem = m;
    
    var fore, back, italic, bold, face;
    // do fixed width
    fore = dialog.currentScheme.getFore('', faceIdentifier(dialog.currentEncoding, 1));
    dialog.fixedColorPickerFore.color = fore;
    back = dialog.currentScheme.getBack('', faceIdentifier(dialog.currentEncoding, 1));
    dialog.fixedColorPickerBack.color = back;
    fore = dialog.currentScheme.getFore('', faceIdentifier(dialog.currentEncoding, 0));
    dialog.propColorPickerFore.color = fore;
    back = dialog.currentScheme.getBack('', faceIdentifier(dialog.currentEncoding, 0));
    dialog.propColorPickerBack.color = back;
    // bold/italic
    propBold = dialog.currentScheme.getBold('', faceIdentifier(dialog.currentEncoding, 0));
    setCheckboxButton(dialog.propBold, propBold);
    fixedBold = dialog.currentScheme.getBold('', faceIdentifier(dialog.currentEncoding, 1));
    setCheckboxButton(dialog.fixedBold, fixedBold);
    propItalic = dialog.currentScheme.getItalic('', faceIdentifier(dialog.currentEncoding, 0));
    setCheckboxButton(dialog.propItalic, propItalic);
    fixedItalic = dialog.currentScheme.getItalic('', faceIdentifier(dialog.currentEncoding, 1));
    setCheckboxButton(dialog.fixedItalic, fixedItalic);
}

function faceIdentifier(encoding, fixed){
    if (fixed) {
        suffix = '_fixed';
    } else {
        suffix = '_proportional';
    }
    return encoding + suffix;
}

function onSampleBlur() {
    //dump("got onSampleBlur\n");
    try {
        dialog.scintilla.scimoz.focus = true;
        dialog.scintilla.scimoz.caretPeriod = 0;
    } catch (e) {
        log.error(e);
    }
}

function onSampleFocus()
{
    try {
        //dump('in focus\n');
        dialog.scintilla.scimoz.caretPeriod = 500;
    } catch (e) {
        log.error(e);
    }
}

function listElement( aListID )
{
    this.listElement = document.getElementById( aListID );
}

listElement.prototype =
{
    clearList:
      function ()
        {
          // removing the menupopup node child of the menulist seems to die!
          // so just remove all *its* child nodes.
          var popup = this.listElement.firstChild;
          while (popup.childNodes.length)
              popup.removeChild( popup.firstChild );
        },

    appendString:
      function ( aString )
        {
          var popupNode;
          popupNode = this.listElement.firstChild;
          if (!popupNode)  {  //clearList may have been called
              popupNode = document.createElement( "menupopup" );
          }

          var itemNode = document.createElement( "menuitem" );
          itemNode.setAttribute( "value", aString );
          itemNode.setAttribute( "label", aString );
          this.listElement.removeAttribute( "disabled" );
          popupNode.appendChild( itemNode );
          this.listElement.appendChild( popupNode );
        },

    appendStrings:
      function ( faces, specs )
        {
          var popupNode;
          popupNode = this.listElement.firstChild;
          if (!popupNode)  {  //clearList may have been called
              log.error("This should never be called - we no longer ever delete the menupopup!!!\n");
              popupNode = document.createElement( "menupopup" );
              this.listElement.appendChild( popupNode );
          }

          faces.sort();
          for( var i = 0; i < faces.length; i++ )
            {
              if( faces[i] == "" )
                {
                  this.listElement.setAttribute( "value", "" );
                  this.listElement.setAttribute( "label", "No fonts available for this language" );
                  this.listElement.setAttribute( "disabled", "true" );
                }
              else
                {
                  var itemNode = document.createElement( "menuitem" );
                  if (specs) {
                    itemNode.setAttribute( "value", specs[faces[i]] );
                  } else {
                    itemNode.setAttribute( "value", faces[i] );
                  }
                  itemNode.setAttribute( "label", faces[i] );
                  // for debug purposes, you may want the fontspec
                  //itemNode.setAttribute( "label", specs[faces[i]] );
                  this.listElement.removeAttribute( "disabled" );
                  popupNode.appendChild( itemNode );
                }
            }
        }
};

