/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/* base partTreeView --

  Knows how to display a hierarchy (possibly flat =) of koIParts, and has notions of adding, renaming, selecting, updating
  
  Known subclasses: partTreeView and snippetTreeView
  
*/


// Globals

// global variable set by the koProjectTree.xml
var gFocusedProjectView = null;

function partTreeView(treeWidget, manager) {
    treeView.apply(this, []);  // inherit from treeView, defined in debugger/treeView.js
    this.log = getLoggingMgr().getLogger("partTreeView");
    this.mSingleSelect = false;  /* Do we want to allow multiple selection or not -- can be overriden in subclasses */
    this.entriesList = []; 
    this.manager = manager;
    this.atomService=Components.classes["@mozilla.org/atom-service;1"].
        getService(Components.interfaces.nsIAtomService); 
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
partTreeView.prototype = new treeView();
partTreeView.prototype.constructor = partTreeView;

partTreeView.prototype.Sort = function(colid) {
    var fieldname = colid;
    if (typeof(this.datapoints[colid]) != 'undefined')
        fieldname = this.datapoints[colid];
    this.manager.sort(fieldname);
}

partTreeView.prototype.clearList = function() {
    //dump("partTreeView.clearList\n");
    this.entriesList = [];
    this.mTotalRows = 0;
    if (this.mTree) {
        this.mTree.rowCountChanged(0,0);
        this.mTree.invalidate();
    }
}

partTreeView.prototype.getSelectedItem = function() {
    var selected = this.mSelection.currentIndex;
    if (typeof(this.entriesList[selected])=='undefined') return null;
     return this.entriesList[selected];
}

partTreeView.prototype.getIndexByPart = function(part) {
    for (var i = 0; i < this.entriesList.length; i++) {
        if (this.entriesList[i] == part) return i;
    }
    return -1;
}

partTreeView.prototype.getPartByIndex = function(index) {
    if (index < 0 || index >= this.entriesList.length) return null;
    return this.entriesList[index];
}

partTreeView.prototype.addToplevelPart = function(/* wrappedPart */ part) {
    window.setCursor("wait");
    this.entriesList[this.entriesList.length] = part;
    this.mTotalRows = this.entriesList.length;
    
    if (part.isContainerOpen) {
        this._toggleAddChildren(this.entriesList.length-1);
        this.mTotalRows = this.entriesList.length;
    }
    
    if (this.mTree) {
        this.mTree.rowCountChanged(0,this.entriesList.length);
        this.mTree.invalidate();
    }
    window.setCursor("auto");
}

partTreeView.prototype.getSelectedToplevelPartId = function() {
    var pi = 0;
    var selected = this.mSelection.currentIndex;
    //dump('current selection is '+selected+'\n');
    for (var i=0; i <= selected; i++) {
        if (this.entriesList[i] && this.entriesList[i].level == 0) {
            if (i == selected) {
                return pi;
            }
            pi++;
        }
    }
    return -1;
}

partTreeView.prototype.getToplevelPartIndexById = function(projectId) {
    var pi = 0;
    for (var i=0; i < this.entriesList.length; i++) {
        if (this.entriesList[i].level == 0) {
            if (pi == projectId) return i;
            pi++;
        }
    }
    return -1;
}

partTreeView.prototype.removePart = function(part) {
    var partIndex = this.getIndexByPart(part);
    var pi = this.getParentIndex(partIndex);
    if (pi >= 0) {
        // remove child from parent
        var parent = this.entriesList[pi];
        if (part) {
            parent.removeChild(part);
        }
    }
    
    // remove child from view
    if (partIndex >= 0) {
        var i;
        var level = this.entriesList[partIndex].level;
        for (i = partIndex + 1; i < this.entriesList.length && level < this.entriesList[i].level; i++) { }
        this.entriesList.splice(partIndex,i-partIndex);

        this.mTotalRows = this.entriesList.length;
        this.mTree.rowCountChanged(0,this.entriesList.length);
        this.mTree.invalidate();
    }
}

/* XXX we should be able to remove this now that removePart is fixed */
partTreeView.prototype.closeToplevelPart = function(Id) {
    var pi = this.getToplevelPartIndexById(Id);
    if (pi >= 0) {
        var project = this.entriesList[pi];
        if (project.isContainerOpen) {
            // just toggle it closed
            this.toggleOpenState(pi);
        }
        // remove project from view
        var fa = this.entriesList.slice(0,pi);
        var fe = this.entriesList.slice(pi+1);
        this.entriesList = fa;
        this.entriesList = this.entriesList.concat(fe);
        this.mTotalRows = this.entriesList.length;
        this.mSelection.currentIndex = -1;
        this.mTree.rowCountChanged(0,this.entriesList.length);
        this.mTree.invalidate();
    }
}

partTreeView.prototype.getCellText = function(row, colID) {
    return 'NIY';
}

partTreeView.prototype.isEditable = function(row, colID) {
    //dump("partTreeView.isEditable("+row+", "+colID+")\n");
    return false;
}

partTreeView.prototype.isContainer = function(index) {
    var res = 0;
    if (index < this.entriesList.length && typeof(this.entriesList[index]) != 'undefined') {
        res = this.entriesList[index].isContainer;
    }
    //dump("partTreeView.isContainer("+index+")=["+res+"]\n");
    return res;
}

partTreeView.prototype.isContainerOpen = function(index) {
    var res = false;
    if (index < this.entriesList.length) {
        return this.entriesList[index].isContainerOpen;
    }
    //dump("partTreeView.isContainerOpen("+index+")=["+res+"]\n");
    return res;
}

partTreeView.prototype.isContainerEmpty = function(index) {
    var res = 0;
    if (index < this.entriesList.length && typeof(this.entriesList[index]) != 'undefined') {
        res = this.entriesList[index].isContainer;
        if (res) {
            res = this.entriesList[index].part.isEmpty();
        }
    }
    return res;
}

  /* long getLevel(in long index); */
partTreeView.prototype.getLevel = function(index) {
    //dump("partTreeView.getLevel("+index+")=["+this.mProjectList[index].level+"]\n");
    if (index < this.entriesList.length && typeof(this.entriesList[index]) != 'undefined') {
        return this.entriesList[index].level;
    }
    return 0;
}

  /* boolean hasNextSibling(in long rowIndex, in long afterIndex); */
partTreeView.prototype.hasNextSibling = function(rowIndex, afterIndex) {
    if (afterIndex < (this.mTotalRows - 1)) {
        var item = this.entriesList[rowIndex];
        for (var i = rowIndex + 1; i < this.mTotalRows && item.level <= this.entriesList[i].level; i++) {
            if (item.level == this.entriesList[i].level) return true;
        }
    }
    return false;
}

  /* long getParentIndex(in long rowIndex); */
  /* Need to signal that there is no parent somehow */
partTreeView.prototype.getParentIndex = function(index) {
    var parent = -1;
    if (index < this.entriesList.length && index > 0 && this.entriesList[index].level > 0) {
        // parent is not going to be correct most of the time, go up the list and compare levels
        var level = this.entriesList[index].level;
        for (var i = index - 1; i >= 0; i--) {
            if (level > this.entriesList[i].level) {
                parent = i;
                break;
            }
        }
    }
    return parent;
}

/* this function is *slower than it should be* if dealing with *deeply* nested structues
    where a lot of them are toggled open because we're splitting the array and recreating
    it on every recursion. */
partTreeView.prototype._toggleAddChildren = function(index) {
    var item = this.entriesList[index];

    if (!item.children) {
        // children have not been added yet
        return;
    }
    //dump('adding '+item.children.length+' rows\n');
    // open the container
    item.sortChildren();
    var fa = this.entriesList.slice(0,index+1);
    var fe = this.entriesList.slice(index+1);
    this.entriesList = fa;
    // insert the children into the array
    this.entriesList = this.entriesList.concat(item.children);
    if (fe.length > 0) this.entriesList = this.entriesList.concat(fe);
    
    // now we have a new index, check if any of the new children are open
    for (var i = index + 1; i < this.entriesList.length && item.level < this.entriesList[i].level; i++) {
        if (this.entriesList[i].level == item.level + 1 && this.entriesList[i].isContainerOpen) {
            this._toggleAddChildren(i);
        }
    }
}

/* void toggleOpenState(in long index); */
partTreeView.prototype.toggleOpenState = function(index,toggle_no_select) {
    if (index >= this.entriesList.length) return;
    var item = this.entriesList[index];
    if (item.isContainer) {
        window.setCursor("wait");
        var firstVisible = this.mTree.getFirstVisibleRow();
        var ci = this.mSelection.currentIndex;
        if (item.isContainerOpen) {
            var i;
            item.isContainerOpen = false;
            var level = item.level;
            for (i = index + 1; i < this.entriesList.length && level < this.entriesList[i].level; i++) { }
            this.entriesList.splice(index+1,i-(index+1));
        } else {
            item.isContainerOpen = true;
            this._toggleAddChildren(index);
        }
        this.mTotalRows = this.entriesList.length;
        this.mTree.rowCountChanged(0,this.entriesList.length);
        this.mTree.invalidate();
        if (typeof(toggle_no_select) == 'undefined' || !toggle_no_select) {
            this.mSelection.currentIndex = ci;
            this.mSelection.select(ci);
        }
        // make sure the first visible row is still the first
        // visible row
        this.mTree.ensureRowIsVisible(firstVisible);
        //dump("partTreeView.toggleOpenState length "+this.mProjectList.length+"\n");
        window.setCursor("auto");
    }
}

  /* void getColumnProperties(in wstring colID, in nsIDOMElement colElt,
     in nsISupportsArray properties); */
partTreeView.prototype.getColumnProperties = function(colID, colElt, properties) {
}
  
  /* void getRowProperties(in long index, in nsISupportsArray properties); */
partTreeView.prototype.getRowProperties = function(index, properties) {
}

  /* void getCellProperties(in long row, in wstring colID, in nsISupportsArray properties); */
partTreeView.prototype.getCellProperties = function(row, colID, properties) {
    return;
}

partTreeView.prototype.getSelectedItems = function() {
    var items = new Array();
    if (this.mSingleSelect) {
        // deselect all other selections except the current one
        this.mSelection.select(this.mSelection.currentIndex);
        items.push(this.entriesList[this.mSelection.currentIndex]);
    } else {
        for (var index=0; index < this.entriesList.length; index++) {
            if (this.mSelection.isSelected(index)) {
                items.push(this.entriesList[index]);
            }
        }
    }
    return items;
}

partTreeView.prototype.selectIndex = function(index) {
    this.mTree.ensureRowIsVisible(index);
    this.mSelection.currentIndex = index;
    this.mSelection.select(index);
}

  /* boolean canDropOn (in long index); */
partTreeView.prototype.canDropOn = function(index) {
    return true;
}

partTreeView.prototype.onDrop = function(event, transferDataSet, session, index) {
    // detect if being draged from a projectviewer, which implies we have a komodo part
    var sourceNode = session.sourceNode;
    var sourceNodeName = null;
    var elementId = null;
    var viewer = null;
    var dragged = null;
    this.log.debug("session.sourceNode " + session.sourceNode);
    if (session.sourceNode)
        this.log.debug("session.sourceNode.nodeName " + session.sourceNode.nodeName);
    this.log.debug("session.numDropItems " + session.numDropItems);
    if (sourceNode && (sourceNode.localName == 'partviewer' ||
                       sourceNode.localName == 'treechildren')) {
        if (sourceNode.localName == 'treechildren') {
            sourceNode = sourceNode.parentNode.parentNode.parentNode;
        }
        try {
            sourceNodeName = sourceNode.localName;
            this.log.debug("  sourceNodeName = " + sourceNodeName);
            elementId = sourceNode.id;
            this.log.debug("  elementId = " + elementId+'\n');
            viewer = document.getElementById(elementId);
            this.log.debug("  viewer = " + viewer.getAttribute('id'));
            dragged = viewer.view.getSelectedItems();
            this.log.debug("  dragged = " + dragged);
        } catch(e) {
            this.log.debug(e);
            // it was not a projectviewer
            sourceNodeName = null;
            elementId = null;
            viewer = null;
            dragged = null;
        }
    }
    this.manager.viewMgr.focus();
    var part;
    // loop through dragged elements
    for ( var i = 0; i < transferDataSet.dataList.length; ++i ) {
        var transferData = transferDataSet.dataList[i];
        // handle the data now
        if (dragged && dragged.length > 0 && sourceNodeName == 'partviewer') {
            // we can only drag from one place at a time, so we should match the returned index!
            var dragpart = dragged[i];
            
                    
            var droppedon = this.getPartByIndex(index);
            var pi = this.getParentIndex(index);
            var parent = this.getPartByIndex(pi);
            var addindex = index;
            if (parent &&
                !droppedon.isContainer &&
                parent.isContainer)
                    addindex = pi;

            if (session.dragAction == Components.interfaces.nsIDragService.DRAGDROP_ACTION_COPY) {
                this.log.debug('copying a part of type '+dragpart.type);
                part = dragpart.part.clone();
                this.manager.addItem(part, addindex);
            } else {
                this.log.debug('moving a part of type '+dragpart.type);
                
                if (droppedon != dragpart &&
                    // if we're dropping on a container, make sure
                    // that we are not already a child of that
                    // container, and that the dropon point is not
                    // a child of the drag part
                    ((droppedon && droppedon.isContainer &&
                     !droppedon.hasChild(dragpart) &&
                     !dragpart.hasChild(droppedon,true))
                    
                    // if what we are dropping on is not a container, then make
                    // sure that it's parent does not already have
                    // the part we're dragging as a direct child
                    || ((!droppedon || !droppedon.isContainer) &&
                        (!parent || !parent.hasChild(dragpart))))
                    ) {
                    
                    // remove the part from it's old parent if we can add it to the new view
                    part = dragpart.part.clone();
                    if (this.manager.addItem(part, addindex))
                        viewer.view.manager.removeItem(dragpart,true);
                }
            }
        } else {
            // XXX this is hardcoded to file or snippet, should be refactored to allow
            // any extension to examine the dropped data.
            var desc = new Object();
            // from DragDrop.js
            this.log.debug('unpacking the dropped data...');
            unpackDroppedData(transferData.dataList[0], desc)
            
            //dump ("data = " + desc.text +'\n');
            if (desc.isDebuggerFile) return false;
            
            if (!desc.isURL) {
                AddSnippetFromTextAtIndex(desc.text,index);
                continue;
            }
            // If we're here, it is a URL
            if (!desc.isFileOrDir) {
                //dump("Adding URL "+desc.text+"\n");
                // is it a wsdl?
                // XXX we need to examine adding wsdl's from protocols other than HTTP
                // also, what if we drop a WSDL file but don't want it to be added as a web service?
                if (desc.text.match(/^\s*http:\/\/.*(\.|\?)wsdl\s*$/i)) {
                    AddWebServiceAtIndex(desc.text,index);
                } else {
                    AddURLAtIndex(desc.text,index);
                }
                continue;
            }
            // if we're here, it's a URL to something we can open (file or folder)
            //dump("is URL\n");
            if (! desc.isExistingFile) {
                // XXX should do something about "missing files?"
                // for now, fall through.
            }
            //dump("is Existing File\n");

            if (desc.isDir) {
                // XXX do we want to add a shortcut, or import from filesystem?
                peFile_addDirectoryShortcut(null,uriparse_URIToLocalPath(desc.text));
            } else if (desc.isFileURL && desc.text.match(/\.kpf$/)) {
                project_openProjectFromURL(desc.text);
            } else {
                // XXX should check that the project doesn't already have the URL






                {
                    part = Components.classes["@activestate.com/koPart?type=file;1"]
                                                    .createInstance(Components.interfaces.koIPart_file);
                }
                part.setStringAttribute('url', desc.text);
                part.setStringAttribute('name', uriparse_baseName(desc.text));
                this.manager.addItem(part, index);
            }
        } 
    } // foreach drag item
    return true;
}

//-------------------------------------------------------------------------------
//  project manager functions called on from extensions

partTreeView.prototype.refresh = function(part) {
    // XXX kind of lame, but works for now
    if (part.isContainer && part.isContainerOpen) {
        var index = this.getIndexByPart(part);
        // close folder
        this.toggleOpenState(index);
        // open folder
        this.toggleOpenState(index);
    }
    this.mTree.invalidate();
}

partTreeView.prototype.selectPart = function(part) {
    var index = this.getIndexByPart(part);
    this.selectIndex(index);
}

partTreeView.prototype.removeSelectedPart = function() {
    var part = this.getSelectedItem();
    if (part) {
        this.removePart(part);
        if (this.mSelection.currentIndex > 0) this.mSelection.currentIndex--;
        this.mSelection.select(this.mSelection.currentIndex);
    }
}

partTreeView.prototype.invalidateRow = function(row) {
    this.mTree.invalidateRow(row);
}

