/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

// start the extension
registeredExtensions[registeredExtensions.length] = new peMacro();

function peMacro() {
    this.name = 'peMacro';
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peMacro.prototype.constructor = peMacro;


peMacro.prototype.init = function() {
}

peMacro.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_editMacro', this);
    extensionManager.registerCommand('cmd_executeMacro', this);
}

peMacro.prototype.registerEventHandlers = function() {
    extensionManager.addEventHandler(Components.interfaces.koIPart_macro,
                                     'ondblclick', this);
    extensionManager.addEventHandler(Components.interfaces.koIPart_macro,
                                     'onUpdatePart', this);
}

peMacro.prototype.registerMenus = function() {
    extensionManager.createMenuItem(Components.interfaces.koIPart_macro,
                                    'Execute Macro',
                                    'cmd_executeMacro',
                                    null,
                                    null,
                                    true);
    extensionManager.createMenuItem(Components.interfaces.koIPart_macro,
                                    'Edit Macro',
                                    'cmd_editMacro');
}

peMacro.prototype.ondblclick = function(item,event) {
    if (item.type != 'macro') return;
    _executeMacro(item.part)
}

peMacro.prototype.supportsCommand = function(command, item) {
    switch (command) {
    case 'cmd_executeMacro':
    case 'cmd_editMacro':
        if (gFocusedProjectView == null) return false;
        var items = gFocusedProjectView.getSelectedItems();
        if (items.length == 1 && items[0].part
            && items[0].part.type == 'macro') {
            return true;
        } else {
            return false;
        }
    default:
        break;
    }
    return false;
}

peMacro.prototype.isCommandEnabled = peMacro.prototype.supportsCommand;

function peMacro_addMacro(item)
{
    var part = Components.classes['@activestate.com/koPart?type=macro;1']
               .createInstance(Components.interfaces.koIPart_macro);
    part.setStringAttribute('filename', '');
    part.setStringAttribute('name', 'New Macro');
    var obj = new Object();
    obj.item = new partWrapper(part, 0);
    obj.task = 'new';
    openWindowUniqueInstance(
        "chrome://jaguar/content/project/macroProperties.xul", 
        "komodo_macroProperties",
        "chrome,close=yes,modal=yes", obj);
    if (obj.res) {
        var index = -1;
        if (typeof(item)=='undefined' || !item) 
            item = gFocusedProjectView.getSelectedItem();
        if (item) index = gFocusedProjectView.getIndexByPart(item);    
        gFocusedProjectView.manager.addItem(part,index);
    }
}

function makeMacroFromFileItem(item) {
    // converting a file into an macro
    var part = Components.classes['@activestate.com/koPart?type=macro;1']
               .createInstance(Components.interfaces.koIPart_macro);
    var filename = uriparse_URIToLocalPath(item.url);
    var osPath = Components.classes["@activestate.com/koOsPath;1"].getService();

    part.setStringAttribute('filename', filename);
    part.setStringAttribute('name', 'Macro: ' + osPath.basename(filename));
    var obj = new Object();
    obj.item = new partWrapper(part, 0);
    obj.task = 'new';
    openWindowUniqueInstance(
        "chrome://jaguar/content/project/macroProperties.xul", 
        "komodo_macroProperties",
        "chrome,close=yes,modal=yes", obj);
    if (obj.res) {
        toolboxMgr.addItem(part);
    }    
}

function makeMacroFromFile(filename) {
    var part = Components.classes['@activestate.com/koPart?type=macro;1']
               .createInstance(Components.interfaces.koIPart_macro);
    var osPath = Components.classes["@activestate.com/koOsPath;1"].getService();
    part.setStringAttribute('filename', filename);
    part.setStringAttribute('name', 'Recorded Macro: ' + osPath.basename(filename));
    toolboxMgr.addItem(part);
    uilayout_ensureToolboxPaneShown();
}

function _executeMacro(part) {
    var fname = part.getStringAttribute('filename');
    var languageRegistry = Components.classes["@activestate.com/koLanguageRegistryService;1"].
          getService(Components.interfaces.koILanguageRegistryService);
    var language = languageRegistry.suggestLanguageForFile(fname).toLowerCase();
    var retval = null;
    var os = Components.classes["@activestate.com/koOs;1"].getService();
    var osPath = Components.classes["@activestate.com/koOsPath;1"].getService();
    if (!osPath.exists(fname)) {
        retval = "The file "+fname+" does not exist"
    }
    if (!osPath.isfile(fname)) {
        retval = fname + " is not a file"
    }
    if (retval == null) {
        var view = gViewMgr.currentView;
        var editor = null;
        if (view) {
            editor = view.scintilla;
        }
        switch (language) {
            case 'javascript':
                try {
                    MacroEvalCode(os.readfile(fname));
                } catch (e) {
                    retval = String(e);
                }
                break;
            case 'python':
            case 'perl':
                retval = part.evalAs(language, document, window, editor);
                break;
            default:
                retval = "Macros written in '"+language+"' aren't supported."
        }
    }
    if (retval) {
        dialog_alert("There was a problem executing the macro: " +
                     part.getStringAttribute('name'), retval);
    }
}

function assertMacroVersion(version) {
    // Here's where we could do something like the following -- which isn't actually
    // needed since the macros from 2.3 appear to work in 2.5 with the three
    // aliases defined in MacroEvalCode
    //if (version != "2") {
    //    throw "Macros from previous versions of Komodo are not supported in this version of Komodo.";
    //}
}

function MacroEvalCode(code) {
    var view = gViewMgr.currentView;
    var editor = null;
    if (view) {
        editor = view.scintilla;
    }
    eval("\
    var komodo;\
    if (typeof(komodo) == 'undefined') {\
        function _Komodo() {\
            this.window = window.QueryInterface(Components.interfaces.nsIDOMWindowInternal);\
        };\
        _Komodo.prototype.constructor = _Komodo;\
        komodo = new _Komodo();\
    };\
    \
    /* For b/w compatibility with Komodo 2.3 */\
    var gLastFocusedViewManager = gViewMgr.currentView;\
    var editor = gLastFocusedViewManager.scimoz;\
    var koGoDoCommand = command_doCommand;\
    ");
    eval(code);    
}

peMacro.prototype.doCommand = function(command) {
    var cmd = null;
    var item = null;
    switch (command) {
    case 'cmd_executeMacro':
        item = gFocusedProjectView.getSelectedItem();
        _executeMacro(item.part);
        break;
    case 'cmd_editMacro':
        item = gFocusedProjectView.getSelectedItem();
        var filename = item.part.getStringAttribute('filename');
        open_openURI(filename);
        break;
    default:
        break;
    }
}

function macro_editProperties(item)
{
    var obj = new Object();
    obj.item = item;
    obj.task = 'edit';
    openWindowMultipleInstance(
        "chrome://jaguar/content/project/macroProperties.xul", 
        "komodo_macroproperties",
        "chrome,close=yes", obj);
}


peMacro.prototype.onUpdatePart = function(item,event)
{
    if (item.type == 'macro') {
        item.renameable = true;
        item.iconstyle['macro'] = 1;
    }
}

function Execute_NamedMacro(name) {
    // this works directly on a part, so the part does not need
    // to be wrapped in order to execute a macro
    var macroPart = toolboxMgr.findPartByTypeAttributeValue('macro','name', name);
    if (!macroPart) {
        macroPart = gProjectManager.findPartByTypeAttributeValue('macro','name', name);
    }
    if (macroPart) {
        _executeMacro(macroPart);
    }
}


