/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/* Komodo Regular Expression Debugger */

function assert(expression) {
    if(!expression) {
        throw "Regex Debugger Assertion!";
    }
}

//---- globals

// References to DOM noes in the dialog.
var dlg = new Object();
var gGlobalPrefs = null;

// Scintillas
var gInput_scimoz;
var gTest_scimoz;
var gInput_scimozEventist;
var gTest_scimozEventist;

//const gInput_scimozFontSize = 12;

// Cached copies of the regex and test string to detect and
// eliminate horrific recursive onModified() calls.
var gRxOldRegex;
var gRxOldTestString;

// Regex info components.
var gRxNodeHighlighter;
var gRxNodeTipGenerator;
var gRxRegexEvaluator;

// The regex evaluation manager component.
var gRxEvalMan;

// Do we need to re-analyze the regex because the user modified
// the regex or test string box?
var gRxDirty;

// An interval timer to keep track the need to update
// the state of the evaluation manager.
var gRxEvalManTimer;

// An object which maintains the current state index and allows
// us to increment it and decrement it without overflowing the
// evaluation manager's state list.
var gRxStateIndexManager;

// UI Elements
var rxNodeTipText;

// Controllers.
var gRxModifierItemsController = null;
var gRxItemsController = null;
var gRxEvalControlItemsController = null;

var rxSuccessLight = null;
var rxFailureLight = null;
var rxClass = null;

// Scintilla styling constants.
var NH_STYLE_BREAK = 1;
var NH_STYLE_CHILD = 2;
var NH_STYLE_PARENT = 3;

var gRxOldHoverPos = -1;
var gRxHoverPos = -1;
var gRxOldNodeTipNodeId = null;

var log = getLoggingMgr().getLogger('rx');
log.setLevel(LOG_INFO);

function regexSciMozEventist() {}
regexSciMozEventist.prototype = {
    onStyleNeeded: function(position) {
        return false;
    },
    onCharAdded: function(ch) {
        return false;
    },
    onSavePointReached: function() {
        return false;
    },
    onSavePointLeft: function() {
        return false;
    },
    onModifyAttemptRO: function() {
        return false;
    },
    onKey: function(ch, modifiers) {
        return false;
    },
    onDoubleClick: function() {
        return false;
    },
    onUpdateUI: function() {
        return false;
    },
    onModified: function(position, modificationType, text, length,
                                 linesAdded, line, foldLevelNow, foldLevelPrev)
    {
        if(gInput_scimoz.text != gRxOldRegex) {
            gRxOldRegex = gInput_scimoz.text;
            gRxDirty = true;
            return true;
        }
        return false;
    },
    onMacroRecord: function(message, wParam, lparam) { 
        return false;
    },
    onMarginClick: function(modifiers, position, margin) {
        return false;
    },
    onNeedShown: function(position, length) {
        return false;
    },
    onPosChanged: function(position) {
        highlightNodeByOffset(position, NH_STYLE_PARENT);
        rxUpdateShortcutMenu(); 
        return false;
    },
    // nsISupports interface
    QueryInterface: function(iid) {
        if (!iid.equals(Components.interfaces.nsISupports) &&
            !iid.equals(Components.interfaces.ISciMozEvents))
        throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
}

function testStringSciMozEventist() {}
testStringSciMozEventist.prototype = new regexSciMozEventist();
testStringSciMozEventist.prototype.constructor = testStringSciMozEventist;
testStringSciMozEventist.prototype.onModified = function(position, modificationType, text, length,
                                 linesAdded, line, foldLevelNow, foldLevelPrev) {
    if(gTest_scimoz.text != gRxOldTestString) {
        gRxOldTestString = gTest_scimoz.text;
        gRxDirty = true;
        return true;
    }
    return false;
}
testStringSciMozEventist.prototype.onPosChanged = function(position) {return false;}


function RxItemsController() {
    //dump("RxItemsController starting up...\n");
}
RxItemsController.prototype = {
    supportsCommand: function(command) {
        switch(command) {
            case 'cmd_loadTestString': 
            case 'cmd_rxRefresh': 
            case 'cmd_rxHelp': 
            return true;
            default: break;
        }
        return false;
    },
    isCommandEnabled: function(command) {
        switch(command) {
            case 'cmd_loadTestString': 
            case 'cmd_rxRefresh': 
            case 'cmd_rxHelp': return true;
            default: break;
        }
        return false;
    },
    doCommand: function(command) {
        switch(command) {
            case 'cmd_loadTestString': this.load_file(); break;
            case 'cmd_rxRefresh':
            gRxDirty = true;
            updateUIState();
            break;
            case 'cmd_rxHelp': this.open_help(); break;
        }
        return true;
    },
    load_file: function() {
        var docSvc = Components.classes['@activestate.com/koDocumentService;1']
                    .getService(Components.interfaces.koIDocumentService);
        var path = filepicker_openFile();
        if (!path) return;
        uri = window.opener.uriparse_pathToURI(path);
        uri = window.opener.uriparse_fixupURI(uri);
        var doc = docSvc.createDocumentFromURI(uri);
        doc.load();
        gTest_scimoz.text = doc.buffer;
    },
    open_help: function()  {
      window.opener.launch_MainHelp('chrome://jaguar/content/doc/komodo/komodo-doc-regex.html')
    }
}

function setupRxShortcutMenu() {
    var menu = document.getElementById("rxShortcutMenuPopup");

    // Get a list of children from gRxNodeHighlighter
    var tokens = {};
    var tokenCount = {};
    var descriptions = {};
    var ranges = {};
    var rangeCount = {};
    var ok = gRxNodeHighlighter.getAllTokens(tokenCount,tokens,descriptions,rangeCount,ranges);
    if (!ok) {
        log.error("Failed to populate shortcut menu.");
        return;
    }

    var mi = document.createElement("menuitem");
    mi.setAttribute("id", "rxShortcutMenuItemNoItems");
    mi.setAttribute("label", "No valid tokens.");
    mi.setAttribute("hidden", false);
    menu.appendChild(mi);
    for (var i=0; i<tokenCount.value; i++) {
        mi = document.createElement("menuitem");
        mi.setAttribute("id", "rxShortcutMenuItem" + i);
        mi.setAttribute("label", tokens.value[i] + ": " +descriptions.value[i]);
        mi.setAttribute("startOffset", ranges.value[2*i]);
        mi.setAttribute("endOffset", ranges.value[2*i + 1]);
        mi.setAttribute("token", tokens.value[i]);
        mi.setAttribute("oncommand", "onRxShortcutMenu(event)");
        menu.appendChild(mi);
    }

    rxUpdateShortcutMenu();
}

var prevStart = -1;
var prevEnd = -1;
function rxUpdateShortcutMenu(event)  {
    var validItems = {};
    var validCount = {};
    var start = gInput_scimoz.selectionStart;
    var end = gInput_scimoz.selectionEnd;
    var id;

    if (start != prevStart || end != prevEnd) {
        var ok = gRxNodeHighlighter.getInsertableTokens(gInput_scimoz.text,
                             gRxModifierItemsController.modifiers,
                             start,
                             end,
                             validCount,
                             validItems
                             );
        var trueCount = false;
        for (var i=0; i<validCount.value; i++) {
            id = "rxShortcutMenuItem" + i;
            SetMenuItemVisibility(id, validItems.value[i]);
            trueCount |= validItems.value[i];
        }
        if (trueCount)
            SetMenuItemVisibility("rxShortcutMenuItemNoItems", false);
        else 
            SetMenuItemVisibility("rxShortcutMenuItemNoItems", true);
        prevStart = start;
        prevEnd = end;
    }
}

// Show/Hide a menu item
// Parameters - 1. menuitem id 2. Boolean to show/hide
function SetMenuItemVisibility(id, showItem)
{
    var item = document.getElementById(id);
    
    if (item) 
        item.setAttribute("hidden", !showItem);
    else
        log.error("SetMenuItemVisibility::item id="+id+" not found\n");
}

function onRxShortcutMenu(event) {
    var mi = event.target;
    var token = mi.getAttribute("token");
    var startOffset = parseInt(mi.getAttribute("startOffset"));
    var endOffset  = parseInt(mi.getAttribute("endOffset"));
    var oldstart = gInput_scimoz.selectionStart;
    var oldend = gInput_scimoz.selectionEnd;
    
    if (gInput_scimoz.selText)
        gInput_scimoz.replaceSel(token);
    else
        gInput_scimoz.insertText(oldstart, token);
    
    gInput_scimoz.selectionStart = oldstart + startOffset;
    gInput_scimoz.selectionEnd = oldstart + endOffset;
}

function RxModifierItemsController() {
    //log.debug("RxModifierItemsController starting up...");
    var mods = ['g', 'm', 'o', 'i', 's', 'x'];
    this.mods = {};
    this.commands = {};
    for (x in mods) {
        var prefName = "rxMod" + mods[x].toUpperCase();
        var cmdName = "cmd_regexMod" + mods[x];
        var checked = false;
        if (gGlobalPrefs.hasBooleanPref(prefName)) {
            checked = gGlobalPrefs.getBooleanPref(prefName);
        }
        this.commands["cmd_regexMod" + mods[x]] = checked;
        this.mods["cmd_regexMod" + mods[x]] = mods[x];
    }
    this.modifiers = this.buildModString();
}
RxModifierItemsController.prototype = {
    supportsCommand: function(command) {
        return this.commands[command] != null;
    },
    isCommandEnabled: function(command) {
        return this.commands[command] == true;
    },
    doCommand: function(command) {
        //log.debug(command);
        if(this.commands[command] == true) {
            this.commands[command] = false;
        } else {
            this.commands[command] = true;
        }

        this.modifiers = this.buildModString();
        window.updateCommands("rx-modifiers");
        gRxDirty = true;
    },
    onEvent: function(event) {
        //log.debug(event);
    },
    buildModString: function() {
        var modString = "";
        for(command in this.commands) {
            if(this.commands[command] == true)
                modString += this.mods[command];
        }
        //log.debug(modString);
        return modString;
    }
};

// Evaluation control items controller.
function RxEvalControlItemsController() {
    //log.debug("RxEvalControlItemsController starting up...");
}
RxEvalControlItemsController.prototype = {
    supportsCommand: function(command) {
        switch(command) {
        case 'cmd_runBackward':
        case 'cmd_stepBack':
        case 'cmd_stepForward':
        case 'cmd_runForward':
            return true;
            break;

        default:
            break; // fall through
        }
        return false;
    },

    isCommandEnabled: function(command) {
        var retval = false;
        switch(command) {
        case 'cmd_runBackward':
            retval = gRxStateIndexManager.canDecIndex();
            return retval;
            break;

        case 'cmd_stepBack':
            retval = gRxStateIndexManager.canDecIndex();
            return retval;
            break;

        case 'cmd_stepForward':
            retval = gRxStateIndexManager.canIncIndex();
            return retval;
            break;

        case 'cmd_runForward':
            retval = gRxStateIndexManager.canIncIndex();
            return retval;
            break;
        }
        return retval;
    },

    doCommand: function(command) {
        switch(command) {
        case 'cmd_runBackward':
            gRxStateIndexManager.
            setIndex(gRxEvalMan.
                        runBackwardUntilBreakpoint(gRxStateIndexManager.currentIndex()));
            //dump("cmd_runBackward: new index is "
            //         + gRxStateIndexManager.currentIndex() + "\n");
            updateEvalState();
            break;

        case 'cmd_stepBack':
            if(gRxStateIndexManager.canDecIndex()) {
                gRxStateIndexManager.decIndex();
                //dump("cmd_stepBack: new index is "
                //       + gRxStateIndexManager.currentIndex() + "\n");
                updateEvalState();
            }
            break;

        case 'cmd_stepForward':
            if(gRxStateIndexManager.canIncIndex()) {
                gRxStateIndexManager.incIndex();
                //dump("cmd_stepForward: new index is "
                //       + gRxStateIndexManager.currentIndex() + "\n");
                updateEvalState();
            }
            break;

        case 'cmd_runForward':
            gRxStateIndexManager.
                        setIndex(gRxEvalMan.
                        runForwardUntilBreakpoint(gRxStateIndexManager.currentIndex()));
            //dump("cmd_runForward: new index is "
            //         + gRxStateIndexManager.currentIndex() + "\n");
            updateEvalState();
            break;
        }

        return true;
    },

    onEvent: function(event) {}
}

function RxEvalStateIndexManager(evalMan) {
    //dump("RxEvalStateIndexManager starting up...\n");
    this.evalMan = evalMan;
    this.reset();
}
RxEvalStateIndexManager.prototype = {
    reset: function() {
        this.stateListIndex = 0;
    },
    
    canIncIndex: function() {
        if (!this.evalMan.runFinished) {
            window.setTimeout('window.updateCommands("rx-eval")', 200);
            return true;
        }
        return this.evalMan.stateCount &&
        this.evalMan.stateCount > this.stateListIndex + 1;
    },
    
    canDecIndex: function() {
        return this.evalMan.stateCount &&
        this.stateListIndex > 0;
    },
    
    currentIndex: function() { return this.stateListIndex; },
    
    incIndex: function() {
        if(this.evalMan.stateCount > this.stateListIndex) {
            this.stateListIndex++;
        }
    },
    
    setIndex: function(index) {
        this.stateListIndex = index;
    },
    
    decIndex: function() {
        if(this.stateListIndex > 0) {
            this.stateListIndex--;
        }
    }
}

function setRxTrafficLights(status) {
    // Set the traffic lights to red or green.
    if (status) {
        rxSuccessLight.setAttribute('class', 'statusbarpanel-iconic  rxSuccessActive');
        rxFailureLight.setAttribute('class', 'statusbarpanel-iconic  rxFailureIdle');
    } else {
        rxFailureLight.setAttribute('class', 'statusbarpanel-iconic  rxFailureActive');
        rxSuccessLight.setAttribute('class', 'statusbarpanel-iconic  rxSuccessIdle');
    }
}

function RxFileDrop() {}
RxFileDrop.prototype = {
    observe : function (subject ,topic, fname) {
        if (topic == 'SciMoz:FileDrop') {
            var scimoz = subject.QueryInterface(Components.interfaces.ISciMoz);
            if (document.getElementById("sci-teststring").ISciMoz == scimoz || document.getElementById("sci-regex").ISciMoz == scimoz) {
                var os = Components.classes['@activestate.com/koOs;1'].getService(Components.interfaces.koIOs);
              scimoz.text = os.readfile(fname);
            }
        }
    }
}


var gDrag_and_drop_listener;

function RxInit() {
    try {
        // Initialise UI Elements
        rxNodeTipText = document.getElementById("rxNodeTipText");
        rxSuccessLight = document.getElementById("rxLightSuccess");
        rxFailureLight = document.getElementById("rxLightFailure");
        
        var input_scintilla = document.getElementById('sci-regex');
        gInput_scimoz = input_scintilla.scimoz;
        input_scintilla.initWithBuffer('', 'Rx');
        gInput_scimoz.viewWS = 1;
        gInput_scimoz.visible = 1;
        gInput_scimoz.caretLineVisible = 0
    
        var test_scintilla = document.getElementById('sci-teststring');
        gTest_scimoz = test_scintilla.scimoz;
        test_scintilla.initWithBuffer('', 'Rx');
        gTest_scimoz.viewWS = 1;
        gTest_scimoz.visible = 1;
        gTest_scimoz.caretLineVisible = 0
        // Add a listener for drag/drop operations.
        gDrag_and_drop_listener = new RxFileDrop();
    
        rxSetMatchoutView(document.getElementById('rx-matchout'),gRxMatchoutView);
    
        // Initialize the regex entry widget.
        gInput_scimozEventist = new regexSciMozEventist();
        gInput_scimoz.hookEvents(gInput_scimozEventist);
        gInput_scimoz.lexer = gInput_scimoz.SCLEX_CONTAINER;
        gInput_scimoz.setCommandUpdateTarget(window);
    
        gInput_scimoz.hScrollBar = false;
        gInput_scimoz.setMarginWidthN(1, 0);
    
        // Hook the test string Scintilla up to its eventist.
        gTest_scimozEventist = new testStringSciMozEventist();
        gTest_scimoz.hookEvents(gTest_scimozEventist);
        gTest_scimoz.lexer = gTest_scimoz.SCLEX_CONTAINER;
        gTest_scimoz.setCommandUpdateTarget(window);
    
        gTest_scimoz.hScrollBar = false;
        gTest_scimoz.setMarginWidthN(1, 0);

        //XXX ugly hack.  For some reason if we do not create the perl debugger listener prior
        //to the rx classes, perl debugging will fail.  See bug #23354
        var perlDbgListener = Components.classes["@activestate.com/koPerlDebuggerListener;1"].createInstance();

        // Load the regex back-end components.
        rxClass = Components.classes['@activestate.com/koRxRegexInfo;1'].createInstance();
        gRxNodeHighlighter = rxClass.QueryInterface(Components.interfaces.koIRxNodeHighlighter);
        assert(gRxNodeHighlighter != null);
    
        gRxNodeTipGenerator = rxClass.QueryInterface(Components.interfaces.koIRxNodeTipGenerator);
        assert(gRxNodeTipGenerator != null);
    
        gRxRegexEvaluator = rxClass.QueryInterface(Components.interfaces.koIRxRegexEvaluator);
        assert(gRxRegexEvaluator != null);
    
        rxClass = Components.classes['@activestate.com/koRxEvalManager;1'].createInstance();
        gRxEvalMan = rxClass.QueryInterface(Components.interfaces.koIRxEvalManager);
        assert(gRxEvalMan != null);
    
        gRxStateIndexManager = new RxEvalStateIndexManager(gRxEvalMan);
    
        // Queue up an eval of the current regex.
        gRxEvalManTimer = window.setInterval(updateUIState, 100);
    
        rxInstallObservers();
        rxInstallControllers();
    
        // Build Shortcuts menu
        setupRxShortcutMenu();
    } catch (e) {
        log.exception(e);
    }
}

function RxOnLoad() {
    try {
        trace_onload();
        command_onload();
        
        dlg.window = document.getElementById('komodo_rx');
        dlg.regexArea = document.getElementById("rx-regex-area");
        dlg.matchVarsArea = document.getElementById("rx-match-vars-area");
        dlg.matchVarsSplitter = document.getElementById("rx-match-vars-splitter");
        dlg.modG = document.getElementById("regexModg");
        dlg.modM = document.getElementById("regexModm");
        dlg.modI = document.getElementById("regexModi");
        dlg.modS = document.getElementById("regexMods");
        dlg.modX = document.getElementById("regexModx");
    
        // Get the preferences
        gGlobalPrefs = Components.classes["@activestate.com/koPrefService;1"].
                       getService(Components.interfaces.koIPrefService).prefs;
    
        RxInit();
        var selection = '';
        if (window.arguments[0] &&
            window.arguments[0].gViewMgr &&
            window.arguments[0].gViewMgr.currentView &&
            window.arguments[0].gViewMgr.currentView.getAttribute('type') == 'editor') {
            selection = window.arguments[0].gViewMgr.currentView.selection;
        }
        if (selection) {
            gInput_scimoz.text = selection;
        } else if (gGlobalPrefs.hasPref("rx-default-regex") &&
                   gGlobalPrefs.hasPref("rx-default-teststring")) {
            gInput_scimoz.text = gGlobalPrefs.getStringPref("rx-default-regex");
            gTest_scimoz.text = gGlobalPrefs.getStringPref("rx-default-teststring");
        }
        gInput_scimoz.emptyUndoBuffer();
        gTest_scimoz.emptyUndoBuffer();
        
        // Set window layout.
        var x = gGlobalPrefs.getLongPref("rxScreenX");
        var y = gGlobalPrefs.getLongPref("rxScreenY");
        var w = gGlobalPrefs.getLongPref("rxOuterWidth");
        var h = gGlobalPrefs.getLongPref("rxOuterHeight");
        var regexAreaHeight = gGlobalPrefs.getLongPref("rxRegexAreaHeight");
        var matchVarsAreaHeight = gGlobalPrefs.getLongPref("rxMatchVarsAreaHeight");
        // XXX Should perhaps validate size/position values to ensure on screen
        //     as done in kombo.js::setupWin().
        dlg.window.setAttribute("screenX", x);
        dlg.window.setAttribute("screenY", y);
        dlg.window.setAttribute("width", w);
        dlg.window.setAttribute("height", h);
        dlg.regexArea.setAttribute("height", regexAreaHeight);
        dlg.matchVarsArea.setAttribute("height", matchVarsAreaHeight);
        document.getElementById('sci-regex').focus();
        // "create" event already fired.
        // Fire a new "load" event our controllers recognize
        window.updateCommands("load");
        window.updateCommands("rx-modifiers");
    } catch (e) {
        log.exception(e);
    }
}

function RxOnUnload() {
    gInput_scimoz.unhookEvents(gInput_scimozEventist);
    gInput_scimozEventist = null;
    gTest_scimoz.unhookEvents(gTest_scimozEventist);
    gTest_scimozEventist = null;

    rxRemoveControllers();
    rxremoveObservers();

    gRxNodeHighlighter = null;
    gRxNodeTipGenerator = null;
    gRxRegexEvaluator = null;
    gRxEvalMan = null;

    // Save window layout.
    gGlobalPrefs.setLongPref("rxScreenX", window.screenX);
    gGlobalPrefs.setLongPref("rxScreenY", window.screenY);
    gGlobalPrefs.setLongPref("rxOuterHeight", window.outerHeight);
    gGlobalPrefs.setLongPref("rxOuterWidth", window.outerWidth);
    gGlobalPrefs.setLongPref("rxRegexAreaHeight",
                              dlg.regexArea.getAttribute("height"));
    gGlobalPrefs.setLongPref("rxMatchVarsAreaHeight",
                              dlg.matchVarsArea.getAttribute("height"));
    gGlobalPrefs.setBooleanPref("rxMatchVarsAreaCollapsed", 
        dlg.matchVarsSplitter.getAttribute("state") == "collapsed");

    // Save current regex and test string as defaults.
    if (gInput_scimoz.textLength != 0) {
        gGlobalPrefs.setStringPref("rx-default-regex", gInput_scimoz.text);
    }
    if (gTest_scimoz.textLength != 0) {
        gGlobalPrefs.setStringPref("rx-default-teststring",
                                    gTest_scimoz.text);
    }

    // Save regex modifiers
    gGlobalPrefs.setBooleanPref("rxModG", dlg.modG.checked);
    gGlobalPrefs.setBooleanPref("rxModM", dlg.modM.checked);
    gGlobalPrefs.setBooleanPref("rxModI", dlg.modI.checked);
    gGlobalPrefs.setBooleanPref("rxModS", dlg.modS.checked);
    gGlobalPrefs.setBooleanPref("rxModX", dlg.modX.checked);
}

// Update the UI in response to an event in the evaluation manager, such
// as moving to the next state in the evaluation state list.
function updateEvalState() {
    try {
        //log.debug("Updating UI in response to a change in the evaluation manager.");
        // Current state will contain an array of state variables.
        var matchOffsets = {};
        var matchOffsetsCount = {};
        var nodeId = {};
        var startOfMatch = {};
        var currentPos = {};
        var closestOpen = {};
    
        try {
            gRxEvalMan.getNodeAt(gRxStateIndexManager.currentIndex(),
                                           nodeId,
                                           startOfMatch,
                                           currentPos,
                                           closestOpen,
                                           matchOffsetsCount,
                                           matchOffsets);
        }
        catch(e) {
            // Okay, so there isn't an active node.
            log.debug("Failed to retrieve state for index " + gRxStateIndexManager.currentIndex());
            log.debug("  because... " + e);
            return;
        }
    
        // Set the group match variables.
        gRxMatchoutView.matchData = matchOffsets.value;
        gRxMatchoutView.setRowCount(matchOffsetsCount.value / 2);
    
        // Highlight the current match ($&)
        scinClearHighlights(gTest_scimoz, NH_STYLE_PARENT);
        scinHighlight(gTest_scimoz, NH_STYLE_PARENT, startOfMatch.value, currentPos.value - 1);
    
        // Try to highlight the current node.
        highlightNodeById(nodeId.value, NH_STYLE_PARENT, false);
    
        // Update the evaluation manager control buttons.
        window.updateCommands("rx-eval");
    } catch (e) {
        log.exception(e);
    }
}

function getNodeIdFromOffset(offset) {
    var nodeId = null;
    
    try {
        nodeId = gRxNodeHighlighter.getNodeId(gInput_scimoz.text,
                                                gRxModifierItemsController.modifiers,
                                                offset);
    } catch(e) { }

    return nodeId;
}

function highlightNodeByOffset(offset, style) {
    var nodeId = getNodeIdFromOffset(offset);
    if(nodeId != null)
        highlightNodeById(nodeId, style, true);
}

function setNodeTipText(statustext) {
    rxNodeTipText.setAttribute("label", statustext);
    rxNodeTipText.setAttribute("tooltiptext", statustext);
}

function setNodeTipById(nodeId) {
    try {
        var nodeTip = gRxNodeTipGenerator.getNodeTip(gInput_scimoz.text,
                                                 gRxModifierItemsController.modifiers,
                                                 nodeId);
        setNodeTipText(nodeTip);
    } catch(e) {
        log.debug("Failed to generate node tip for node id " + nodeId + "\n");
    }
}

function highlightNodeById(nodeId, style, highlightGroupMatches) {
    var a = {};
    var b = {};
    try {
        scinClearAllHighlights(gInput_scimoz);
        if (!nodeId) return;
        gRxNodeHighlighter.getNodeRange(gInput_scimoz.text,
                                    gRxModifierItemsController.modifiers,
                                    nodeId, 0, a, b);

        scinHighlight(gInput_scimoz, style, a.value, b.value);

        gRxNodeHighlighter.getChildNodesRange(gInput_scimoz.text,
                                          gRxModifierItemsController.modifiers,
                                          nodeId, a, b);
        if(!(a.value == 0 && b.value == 0)) {
            scinHighlight(gInput_scimoz, NH_STYLE_CHILD, a.value, b.value);
        }

        // That's all fine and dandy; but if it's a group, we do more:
        // 1. Highlight the other group
        // 2. Highlight the areas in the target string that matched this
        //    group.

        var gid = gRxNodeHighlighter.getGroup(gInput_scimoz.text, 
                                              gRxModifierItemsController.modifiers,
                                              nodeId);
        if (gid != 0) {
            var mid = gRxNodeHighlighter.getMatchingNode(gInput_scimoz.text,
                                                         gRxModifierItemsController.modifiers,
                                                         nodeId);
            gRxNodeHighlighter.getNodeRange(gInput_scimoz.text,
                                            gRxModifierItemsController.modifiers,
                                            mid, 0, a, b);
            scinHighlight(gInput_scimoz, style, a.value, b.value);

            // This feature may want to be in a preference:
            var globalMode = (gRxModifierItemsController.modifiers.indexOf('g') >= 0);
            var matchOffsetsCount = {};
            var matchOffsets = {};
            var groupCount = {};
            var errorString = {};
            var matched = gRxRegexEvaluator.
                evalRegex(gInput_scimoz.text,
                          gRxModifierItemsController.modifiers,
                          gTest_scimoz.text,
                          groupCount,
                          matchOffsetsCount,
                          matchOffsets,
                          errorString);

            if(highlightGroupMatches) {
                scinClearAllHighlights(gTest_scimoz);
                for (var i=2*gid;
                     i<matchOffsetsCount.value; 
                     i+=2*groupCount.value) {
                    var start = matchOffsets.value[i];
                    var end = matchOffsets.value[i+1];
                    if (start != -1 && end != -1 && start <= end)
                        scinHighlight(gTest_scimoz, style, start, end);
                }
            }
        }
    }
    catch(e) {
        log.debug("highlightNodeById: Can't highlight node " + nodeId + "\n");
        log.exception(e);
        return;
    }
    setNodeTipById(nodeId);
}

// Clear all the highlights from a Scintilla.
function scinClearAllHighlights(scin) {
    try {
        scin.clearDocumentStyle();
    } catch (e) {
        log.exception(e);
    }
}

// Clear all the highlights from a Scintilla which are
// in a particular style.
function scinClearHighlights(scin, style) {
    if(scin.length > 0) {
        try {
            scin.startStyling(0, style);
            scin.setStyling(scin.length, 0);
        } catch (e) {
            log.exception(e);
        }
    }
}

// Highlight some part of a Scintilla widget with some style.
function scinHighlight(scin, style, startOffset, endOffset) {
    try {
        // Don't bother highlighting if the Scintilla has zero length or
        // if the end and start offsets are the same.
        if(scin.length == 0) {
            return;
        }
    
        if(startOffset > endOffset) {
            var tempOffset = startOffset;
            startOffset = endOffset;
            endOffset = tempOffset;
        }
    
        log.debug("Highlighting from " + startOffset + " to " + endOffset +
                 " with style " + style + " in scin " + scin);
        
        // Adjust the starting and ending offsets to prevent 
        var scinLength = scin.length;
        if(startOffset > scinLength - 1) {
            startOffset = scinLength - 1;
        }
        if(startOffset < 0) {
            startOffset = 0;
        }
    
        if(endOffset > scinLength - 1) {
            endOffset = scinLength;
        }
        if(endOffset < 0) {
            endOffset = 0;
        }
        
        scin.startStyling(startOffset, style);
        scin.setStyling(endOffset - startOffset + 1, style);
    } catch (e) {
        log.exception(e);
    }
}

// Once in a while, check to see whether the user has modified either
// the regular expression string or the test string.
// If either of these have been modified, as indicated by the
// gRxDirty flag, then reset the evaluation manager and trigger
// a command update of the regex evaluation control commands.
function updateUIState() {
    try {
        if (!gRxDirty) return;
        if (gInput_scimoz.text == '' || gTest_scimoz.text == '') {
            // Need to clear the matches but _not_ evaluate, since
            // that causes a 5-10 second hang.
            gRxMatchoutView.setRowCount(0);
            scinClearHighlights(gTest_scimoz, NH_STYLE_PARENT);
            return;
        }
        if (!updateLinting()) return;
        gRxDirty = false;
        gRxEvalMan.initEval(gInput_scimoz.text,
                          gTest_scimoz.text,
                          gRxModifierItemsController.modifiers);
        gRxStateIndexManager.reset();
        updateMatches();
    
        // Give the evaluation manager some time to gather a state list.
        window.setTimeout('window.updateCommands("rx-eval");', 100);
    } catch (e) {
        log.exception(e);
    }
}

function onSciRegexMouseMove(event) {
    try {
        var x = event.clientX - event.target.boxObject.x;
        var y = event.clientY - event.target.boxObject.y;
        gRxHoverPos = gInput_scimoz.positionFromPoint(x, y);
        
        if(gRxOldHoverPos != gRxHoverPos) {
            var nodeId = getNodeIdFromOffset(gRxHoverPos);
            if (!nodeId) return;
            if(gRxHoverPos >= 0 && nodeId != gRxOldNodeTipNodeId) {
                highlightNodeById(nodeId, NH_STYLE_PARENT, true);
                gRxOldNodeTipNodeId = nodeId;
            }
            gRxOldHoverPos = gRxHoverPos;
        }
    } catch (e) {
        log.exception(e);
    }
}

function updateLinting() {
    try {
        // Perform syntax error linting (crude!)
        var errorString = {};
        var isValid = gRxRegexEvaluator.isValidRegex(gInput_scimoz.text,
                                         gRxModifierItemsController.modifiers,
                                         errorString);
        if(!isValid) {
            gInput_scimoz.indicSetStyle(2, gInput_scimoz.INDIC_SQUIGGLE);
            gInput_scimoz.indicSetFore(2, 0x0000ff);
            scinHighlight(gInput_scimoz, gInput_scimoz.INDIC2_MASK,
                        0, gInput_scimoz.length - 1);
    
            setRxTrafficLights(false);
            setNodeTipText(errorString.value);
            scinClearAllHighlights(gTest_scimoz);
        } else {
            gInput_scimoz.startStyling(0, 0xff);
            gInput_scimoz.setStyling(gInput_scimoz.length - 1, 0);
        }
        return isValid;
    } catch (e) {
        log.exception(e);
    }
    return false;
}

// Perform a match against the test string and update the UI
// in response to the match.
function updateMatches() {
    try {
        scinClearAllHighlights(gTest_scimoz);
        var matchOffsetsCount = {};
        var matchOffsets = {};
        var groupCount = {};
        var errorString = {};
        var matched = gRxRegexEvaluator.
            evalRegex(gInput_scimoz.text,
                      gRxModifierItemsController.modifiers,
                      gTest_scimoz.text,
                      groupCount,
                      matchOffsetsCount,
                      matchOffsets,
                      errorString);
    
        if(matched) {
            setRxTrafficLights(true);
            if (groupCount.value == 1)
                setNodeTipText("Match succeeded: 1 group");
            else
                setNodeTipText("Match succeeded: " + groupCount.value + " groups");
    
            if(matchOffsetsCount.value > 1) {
                scinHighlight(gTest_scimoz,
                              NH_STYLE_PARENT,
                              matchOffsets.value[0],
                              matchOffsets.value[1]);
                gRxMatchoutView.matchData = matchOffsets.value.slice(2);
                gRxMatchoutView.groupCount = groupCount.value;
                gRxMatchoutView.setRowCount((matchOffsetsCount.value - 2) / 2);
            }
            else {
                log.error("Insufficient match offsets.");
                log.error("matchOffsetsCount: " + matchOffsetsCount.value);
                log.error("matchOffsets:      " + matchOffsets.value);
            }
        } else {
            gRxMatchoutView.setRowCount(0);
            setRxTrafficLights(false);
            setNodeTipText("No matches found");
        }
    } catch (e) {
        log.exception(e);
    }
}

// The view object for the match variables tree widget.
var gRxMatchoutView = ({
    // nsITreeView
    rowCount : 0,
    getRowProperties : function(i, prop) {
        // prop.AppendElement(this.smile);
    },
    getColumnProperties : function(index, prop) {},
    getCellProperties : function(index, prop) {},
    isContainer : function(index) {return false;},
    isSeparator : function(index) {return false;},
    tree : null,
    setTree : function(out) { this.tree = out; },
    getImageSrc : function() {return null;},
    getCellText : function(i, col) {
        var globalMode =
            (gRxModifierItemsController.modifiers.indexOf('g') >= 0);
        var text = "";
        switch(col){
        case "rx-matchout-var":
            if(globalMode) {
                var m = Math.round(i / this.groupCount + 0.5);
                var v = (i % this.groupCount) + 1;
                text = "$" + v + " (match " + m + ")";
            }
            else {
                text = "$" + (i + 1);
            }
            break;
        case "rx-matchout-value":
            text = gTest_scimoz.text.
                substring(this.matchData[2*i],
                          this.matchData[2*i + 1] + 1);
            break;
        default:
            text = "XXX in " + col + " and " + i;
        }
        return text;
    },
    performAction : function(action) {
    },
    cycleHeader : function(index) {
    },
    selectionChanged : function() {
        if (this.selection.currentIndex >= 0 && this.matchData
            && this.matchData.length > 2 * this.selection.currentIndex) {
            var startOffset = this.matchData[2 * this.selection.currentIndex];
            var endOffset = this.matchData[2 * this.selection.currentIndex + 1];
            gTest_scimoz.gotoPos(startOffset);
            scinClearAllHighlights(gTest_scimoz);
            scinHighlight(gTest_scimoz, NH_STYLE_PARENT, startOffset,
                          endOffset);
        }
    },
    
    // Private stuff
    matchData : null,
    groupCount : 0,
    isSorted: function() {return false;},
    setRowCount : function(rowCount) {
        this.rowCount = rowCount;
        this.tree.rowCountChanged(0, this.rowCount);
        this.tree.invalidate();
    }
});
 
function rxSetMatchoutView(tree, v) {
    try {
        tree.treeBoxObject.QueryInterface(Components.interfaces.nsITreeBoxObject).view = v;
    } catch (e) {
        log.exception(e);
    }
}

function rxInstallObservers() {
    try {
        var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                                                getService(Components.interfaces.nsIObserverService);
    
        observerSvc.addObserver(gDrag_and_drop_listener, "SciMoz:FileDrop",false);
    } catch (e) {
        log.exception(e);
    }
}

function rxremoveObservers() {
    try {
        var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                                                getService(Components.interfaces.nsIObserverService);
    
        observerSvc.removeObserver(gDrag_and_drop_listener, "SciMoz:FileDrop");    
    } catch (e) {
        log.exception(e);
    }
}

function rxInstallControllers() {
    try {
        gRxEvalControlItemsController = new RxEvalControlItemsController();
        window.controllers.insertControllerAt(0, gRxEvalControlItemsController);
        
        gRxItemsController = new RxItemsController();
        window.controllers.insertControllerAt(0, gRxItemsController);
        
        gRxModifierItemsController = new RxModifierItemsController();
        window.controllers.insertControllerAt(0, gRxModifierItemsController);
    } catch (e) {
        log.exception(e);
    }
}

function rxRemoveControllers() {
    try {
        window.controllers.removeController(gRxModifierItemsController);
        window.controllers.removeController(gRxItemsController);
        window.controllers.removeController(gRxEvalControlItemsController);
        //  window.controllers.removeController(gRxTokenMenuController);
    
        gRxModifierItemsController = null;
        gRxItemsController = null;
        gRxEvalControlItemsController = null;
    } catch (e) {
        log.exception(e);
    }
}

function rxUpdateCommands()
{
    try {
        // Ensure any buffer sensitive commands are updated.
        window.updateCommands("buffer-focus");
        window.updateCommands("debug");
         // The selection may have changed.
        window.updateCommands("select");
        // The save and undo state of the file will be different.
        window.updateCommands("save");
        window.updateCommands("undo");
    } catch (e) {
        log.exception(e);
    }
}

