#!python
# Copyright (c) 2000-2001 ActiveState Corporation.
# See the file LICENSE.txt for licensing information.

import os
import xml.sax
from xml.dom import minidom
from xpcom import components
from xpcom import _xpcom  #XXX for getService hack
import shutil, string
import uriparse
import sys
import logging

log = logging.getLogger("koBookmark")


XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul"
WSBK_EXT = ".asbk"
DOC_EXT = ".html"

def koDumpDOM(node, level=0):
    print " " * 2 * level + '<' + node.nodeName
#    for i in node.attributes:
 #       print ' '*(2*level + 4) + node.attributes[i].nodeName + ' = ' + node.attributes[i].nodeValue
    if not node.childNodes:
        print ' '*2*level +'/>'
    else:
        print ' '*2*level +'>'
        for child in node.childNodes:
            koDumpDOM(child, level + 1);
        print ' '*2*level +'</' + node.nodeName + '>'
        
def normalize(root):

    """Join adjacent text nodes in a DOM tree
    
    This was stolen from Python 2.1's implementation of minidom, with
    adaptations to make it a stand alone function.  Once we upgrade
    the siloed Python, this should die.
    
    """
    
    L = []
    for child in root.childNodes:
        if child.nodeType == minidom.Node.TEXT_NODE:
            data = child.data
            if data and L and L[-1].nodeType == child.nodeType:
                # collapse text node
                node = L[-1]
                node.data = node.nodeValue = node.data + child.data
                node.nextSibling = child.nextSibling
                child.unlink()
            elif data:
                if L:
                    L[-1].nextSibling = child
                    child.previousSibling = L[-1]
                else:
                    child.previousSibling = None
                L.append(child)
            else:
                # empty text node; discard
                child.unlink()
        else:
            if L:
                L[-1].nextSibling = child
                child.previousSibling = L[-1]
            else:
                child.previousSibling = None
            L.append(child)
            if child.nodeType == minidom.Node.ELEMENT_NODE:
                normalize(child)
    root.childNodes[:] = L

_koDirs = None

def _getKoDirs():
    global _koDirs
    if _koDirs is None:
        _koDirs = components \
                           .classes["@activestate.com/koDirs;1"] \
                           .getService()
    return _koDirs

def _getUserDataDir():
    return _getKoDirs().userDataDir

def _get_rootDirectory():
    wsdir = os.path.join(_getUserDataDir(), "webservices")
    if not os.path.exists(wsdir):
        try:
            os.makedirs(wsdir)
        except OSError:
            log.error("couldn't create directory %s in "
                      "koBookmarkManager._get_rootDirectory", wsdir)
            return ''
    bmdir = os.path.join(wsdir, "bookmarks")
    if not os.path.exists(bmdir):
        try:
            os.makedirs(bmdir)
        except OSError:
            log.error("couldn't create directory %s in "
                      "koBookmarkManager._get_rootDirectory", bmdir)
            return ''
    return bmdir

def _MakeParentIdFromPath(path):
    ud = _getUserDataDir()
    ud = string.replace(ud, "\\", "/")
    path = string.replace(path, "\\", "/")
    if ud[len(ud)-1] != "/": ud += "/"
    l = len(ud)
    id = path[l:]
    id = string.replace(id, "/", ":")
    #print "_MakeParentIdFromPath(%s)=%s"%(path,id)
    return id
    

class koBookmarkManager:
    _com_interfaces_ = [components.interfaces.koIBookmarkManager]
    _reg_desc_ = "Komodo Web Services Bookmark Manager"
    _reg_clsid_ = "{70E726C1-28FE-4f3d-823E-517FE9445D97}"
    _reg_contractid_ = "@activestate.com/koBookmarkManager;1"

    extension = WSBK_EXT
    _wsRootDir = ""
    
    def __init__(self):
        # this is to create it if it does not exist
        self.wsRootDir = self.get_rootDirectory() 
        
    def get_rootDirectory(self):
        if not self._wsRootDir:
            self._wsRootDir = _get_rootDirectory()
        return self._wsRootDir
    
    def open(self, filename):        
        #TODO: handle parsing errors
        return koBookmark(filename)
    
    def createFolder(self, dir):
        head, tail = os.path.split(dir)
        name = tail
        parentID = _MakeParentIdFromPath(head)
        #print "createFolder name=",name," parentID=",parentID
        if not os.path.exists(dir):
            os.mkdir(dir)
    
    def _removeDir(self, dir):
        # this only removes the directory from the RDF, not from disk
        # removal from disk is handled in remove()
        if os.path.isdir(dir):
            #print "_removeDir(%s)"%(dir)  # remove this directory
            head, tail = os.path.split(dir)
            name = tail
            parentID = _MakeParentIdFromPath(head)
            #print "_removeDir: name=[%s] parentID=[%s]"%(name,parentID)
    
    def _removeDirectory(self, arg, dir, files):
        #this removes files in a directory from disk and rdf
        for file in files:
            f, e = os.path.splitext(file)
            if e <> WSBK_EXT: continue
            filename = os.path.join(dir, file)
            if os.path.isfile(filename):
                #print "_removeDirectory removing ",filename
                bm = koBookmark(filename)
                bm.remove()
            #else:
            #    self.remove(filename)
        self._removeDir(dir)
    
    def remove(self, filename):
        # delete all the files related to bmname, remove from rdffile
        if os.path.isdir(filename):
            # rdf api will not recursively delete from the rdf, we must do it
            os.path.walk(filename, self._removeDirectory, self)
            # remove this directory
            #self._removeDir(filename)
            shutil.rmtree(filename,1)
        else:
            # remove this file, and associated doc files
            bm = koBookmark(filename)
            bm.remove()
    
    def populateMenu(self, document, menupopup):
        if not self._wsRootDir:
            self.get_rootDirectory()
        menumap = {self._wsRootDir: menupopup}
        self._add_children(document, self._wsRootDir, menumap)

    def _add_children(self, document, curdir, menumap):
        for thing in os.listdir(curdir):
            fullpath = os.path.join(curdir, thing)
            if os.path.isdir(fullpath):
                newmenu = document.createElementNS(XUL_NS, 'menu')
                newmenu.setAttribute('label', thing)
                newmenu.setAttribute('id', fullpath.replace(' ',''))
                newmenupopup = document.createElementNS(XUL_NS, 'menupopup')
                newmenu.setAttribute('id', fullpath+'popup')
                #print menuitem.getAttribute('id')
                newmenu.appendChild(newmenupopup)
                menumap[fullpath] = newmenupopup
                menumap[curdir].appendChild(newmenu)
                self._add_children(document, fullpath, menumap)
            else:
                menuitem = document.createElementNS(XUL_NS, 'menuitem')
                menuitem.setAttribute('id', fullpath.replace(' ',''))
                #print menuitem.getAttribute('id')
                menuitem.setAttribute('label', os.path.splitext(thing)[0])
                menuitem.setAttribute('oncommand', 'koInsertWSUsage('+fullpath+')')
                menumap[curdir].appendChild(menuitem)
        #print map(koDumpDOM, menumap.values())
        return 1


            
class koBookmark:
    _com_interfaces_ = [components.interfaces.koIBookmark]
    _reg_desc_ = "Komodo Web Services Bookmark"
    _reg_clsid_ = "{5A4E9A34-E28B-4e8b-B854-3CB136AB612B}"
    _reg_contractid_ = "@activestate.com/koBookmark;1"

    def __init__(self, filename = ""):
        self.filename = ""
        self._docfile = ""
        fn = ""
        
        if filename:
            self.filename = str(filename)   # minidom doesn't know how to handle Unicode filenames in Python 2.0.
            # split the filename into parts and build the help filename
            basename, fn = os.path.split(self.filename)
            fn, ext = os.path.splitext(fn)
            self._docfile = os.path.join(basename, fn+DOC_EXT)
            
        if os.path.exists(filename):
            self._dom = minidom.parse(self.filename)
            normalize(self._dom)
            
        else:
            self._dom = self._buildBlankDocument()
            
        if fn:
            self.set_name(fn)

        # Save the bookmark
        if not self.filename:
            self.filename = os.path.join(_get_rootDirectory(),'temp'+WSBK_EXT)
    
    def get_url(self):
        return self._get("url")
    
    def set_url(self, value):
        self._set("url", value)

    def get_name(self):
        return self._get("name")
    
    def set_name(self, value):
        self._set("name", value)

        dir = os.path.dirname(self.filename)
        self.parentID = _MakeParentIdFromPath(dir)
        #print "parentID=",self.parentID

        newfilename = os.path.join(os.path.dirname(self.filename), value + WSBK_EXT)
        newdocFileName = os.path.join(os.path.dirname(self.filename), value + DOC_EXT)
        if newfilename != self.filename:
            try:
                os.rename(self.filename, newfilename)
            except OSError, e:
                import errno
                if errno.errorcode[e.errno] == 'ENOENT':
                    # source file does not exist, we don't care
                    pass
                else:
                    raise
                
            self.filename = newfilename
            
            try:
                os.rename(self._docfile, newdocFileName)
            except OSError, e:
                import errno
                if errno.errorcode[e.errno] == 'ENOENT':
                    # source file does not exist, we don't care
                    pass
                else:
                    raise

            self._docfile = newdocFileName

    def get_shortname(self):
        return self._get("shortname")
    
    def set_shortname(self, value):
        self._set("shortname", value)
        
    def get_description(self):
        return self._get("description")
    
    def set_description(self, value):
        self._set("description", value)
    
    def save(self, name, localOnly):
        # XXX this code needs a serious revisiting. The possible error conditions should be
        # carefully examined and handled appropriately.
        
        """
        Flush any changes to disk.
        """
        """
        save the documentation
        """
        import WShelper
        if localOnly:
            cacheMode = WShelper.FORCE_CACHE
        else:
            cacheMode = WShelper.FORCE_FETCH

        try:
            # XXX This is a cheap hack to ensure that the user cannot create a bookmark without
            # the WSDL file being cached.
            WShelper.ExtractWSDL(self.get_url(), cacheMode)
        except IOError, e:
            return 0, 'The bookmark "%s" could not be saved because the WSDL file "%s" could not be retrieved [%r].'\
                   % (name, self.get_url(), e.strerror)
        
        # Save the bookmark

        try:
            self.set_name(name)
        except OSError, e:
            import errno
            if errno.errorcode[e.errno] == 'EEXIST':
                return 0,   'The bookmark "%s" could not be saved because the file already exists [%s].' % (name, e.strerror)

        try:
            bookmarkFile = open(self.filename, "w")
            self._dom.writexml(bookmarkFile)
            bookmarkFile.close()
        except IOError, e:
            return 0, 'The bookmark "%s" could not be saved because the file "%s" could not be written to [%r]' % (name, self.filename, e.strerror)
        except Exception, e:
            # log the traceback
            return 0, 'The bookmark "%s" could not be saved due to an internal error [%r]' % (name, e.args)
        
        # read the header
        try:            
            fname = os.path.join(_getKoDirs().mozBinDir, "chrome","jaguar","content","webservices","template.html")
            file = open(fname, "r")
            template = file.read()
            file.close()
        except IOError, e:
            import traceback
            traceback.print_exception(*sys.exc_info())

            return 1,   'An error occured while reading the Komodo WebServices documentation template [%r]. '\
                        'The bookmark "%s" has been saved but the documentation and intellisense will not be available.' \
                        % (e.args, name)

        template = template.replace('%%%TITLE%%%', os.path.splitext(os.path.basename(self._docfile))[0])
        wsdlfile = str(self.get_url())


        try:
            doc = WShelper.WSDL_to_HTML(wsdlfile, self.get_description(), WShelper.FORCE_CACHE)
        except IOError, e:
            return 1,   'An error occured while reading the WSDL file [%s]. '\
                        'The bookmark "%s" has been saved but the documentation and intellisense will not be available.'\
                        % (e.strerror, name)
        #except SGMLSyntaxError:
        #    return 1,   'The WSDL file contains XML errors. '\
        #                'The bookmark "%s" has been saved but the documentation and intellisense will not be available.'\
        #                % name
        except Exception, e:
            import traceback
            traceback.print_exception(*sys.exc_info())
            return 1,    'The WSDL file could not be parsed by Komodo [%r]. '\
                         'The bookmark "%s" has been saved but the documentation and intellisense will not be available.'\
                         % (e.args, name)
        
        try:
            htmlfile = string.replace(template, "<!-- WEBSVCHELP -->", doc)
            if doc:
                file = open(self._docfile, "w")
                file.write(htmlfile)
                file.close()

        except Exception, e:
            import traceback
            traceback.print_exception(*sys.exc_info())
            return 1, 'The bookmark has been saved but the documentation and intellisense '\
                      'will not be available due to an internal error [%r].' % e.args
        return 1, ""
    
    def remove(self):
        try: os.remove(self.filename)
        except OSError: pass  # they may have screwed w/ the file themselves.
        try: os.remove(self._docfile)
        except OSError: pass  # they may have screwed w/ the file themselves.
    
    def _get(self, tagName):
        targetNode = self._dom.getElementsByTagName(tagName)[0]
        if targetNode.hasChildNodes():
            return targetNode.firstChild.data
        else:
            return ""
        
    def _set(self, tagName, value):
        targetNode = self._dom.getElementsByTagName(tagName)[0]
        if targetNode.hasChildNodes():
            targetNode.removeChild(targetNode.firstChild)
        if value:
            textNode = self._dom.createTextNode(value)
            targetNode.appendChild(textNode)
        
    def _buildBlankDocument(self):
        doc = minidom.Document()
        shortnameElement = doc.createElement("shortname")
        urlElement = doc.createElement("url")
        descriptionElement = doc.createElement("description")
        nameElement = doc.createElement("name")
        bookmarkElement = doc.createElement("bookmark")
        bookmarkElement.setAttribute("version", "1.0")
        bookmarkElement.appendChild(shortnameElement)
        bookmarkElement.appendChild(urlElement)
        bookmarkElement.appendChild(descriptionElement)
        bookmarkElement.appendChild(nameElement)
        doc.appendChild(bookmarkElement)
        return doc

    def generateFromWSDL(self):
        # given that the URL has been set, parse it and fill in the rest of the info
        # for this class
        wsdlfile = str(self.get_url())
        if not wsdlfile: return 0
        # parse the wsdl url
        #print "generating from ",wsdlfile
        import WShelper
        try:
            wsdl = WShelper.ExtractWSDL(wsdlfile, cache=WShelper.FORCE_FETCH)
        except IOError, e:
            return 0, e.strerror
        except OSError, e:
            return 0, e.strerror
        #except SGMLSyntaxError:
        #    return 0, "Error parsing WSDL file"
        
        if wsdl:
            self.set_name(wsdl.serviceName)
            self.set_shortname(wsdl.serviceName[wsdl.serviceName.rfind('.')+1:])
            self.set_description(wsdl.documentation)
            return 1, ""
        return 0, "Unknown error parsing WSDL file"
    
    def refreshWSDL(self):
        """Update the local WSDL cache and renew all dependant information"""
        return self.generateFromWSDL()
