# Copyright (c) 2000-2001 ActiveState Corporation.
# See the file LICENSE.txt for licensing information.

"""
This script calls a Komodo process for remote debugging.
"""
import errno
from korpc import Server
import sys
import kdb
import os
import threading
from socket import *
import string

_expected_connection_errors = [errno.ECONNREFUSED]
if hasattr(errno, "WSAETIMEDOUT"):
    _expected_connection_errors.append(errno.WSAETIMEDOUT)

requestor = None
def _prepareDebugger(komodo_hostname=None, komodo_port=None):
    global requestor
    if requestor is None:
        requestor, connection_port, cookie = kdb.ConnectToListener(komodo_hostname, komodo_port)

# Break into the Komodo debugger.  A "hard-coded" breakpoint - always forces a break
# into the debugger UI.
def brk(komodo_hostname = None, komodo_port = None):
    try: _prepareDebugger(komodo_hostname, komodo_port)
    except error, e:
        # if we can't connect to Komodo, keep going as if nothing happened
        # we may want to print out an error message
        (en, message) = e
        if en not in _expected_connection_errors:
            raise e
    else: requestor.brk()

doneFirstPrepare = 0

# A function that "prepares" the stack frame for debugging.  This is
# mainly used by things like the xpcom entry point, to ensure that
# _all_ xpcom components are run with debugger hooks enabled - thereby
# enabling breakpoints. In general, calling this function does _not_
# force the debugger UI up
# EXCEPT: First time it is called, the UI _will_ be presented, due to
# a limitation in Komodo - this is your only opportunity to actually
# _set_ the breakpoints in a remote process.

# In the ideal world, you should be able to set a breakpoint asynchronously
# into running Python code.  When that day somes, this can die.
def prepare():
    global doneFirstPrepare
    if not doneFirstPrepare:
        doneFirstPrepare = 1
        _prepareDebugger()    
        requestor.brk()
        # NOTE: You are breaking here to give you the opportunity
        # to setup the debugging session - ie, set break-points, etc.
        # Press <Go> when you are ready.
        pass
    else:
        # Almost duplicate of set_break(), except we just continue!
        try:
            1 + ''
        except:
            frame = sys.exc_info()[2].tb_frame.f_back
        while frame:
            frame.f_trace = self.trace_dispatch
            self.botframe = frame
            frame = frame.f_back
        requestor.kdb.set_continue()
        sys.settrace(requestor.kdb.trace_dispatch)

# The entry point when run as a script.  This will execute the script named
# on the commandline under the debugger.
def main(args):
    if len(args)>=3:
        komodo_hostname, komodo_port, script=args[:3]
        try:
            komodo_port = int(komodo_port)
        except (ValueError, TypeError):
            print "The specified port ('%s') must be a valid integer" % (komodo_port,)
            sys.exit(99)
        args=args[3:]
        script_args = [script]+args
        try: _prepareDebugger(komodo_hostname, komodo_port)
        #try: _prepareKomodoDebugger(komodo_hostname, komodo_port)
        except error, e:
            (en, message) = e
            if en in _expected_connection_errors:
                print \
"""Could not connect to Komodo.  Please check that Komodo is listening for
Python and that hostname and port are correct."""
            else:
                raise e
        else: requestor.start(script_args)
    else:
        print "Usage: callkomodo.py hostname port script [arguments]"

if __name__=="__main__":
    main(sys.argv[1:])
