"""Test Project functionality in Komodo."""

import os, sys
import unittest
from xpcom import components, ServerException, COMException, nsError
from xpcom.server import WrapObject, UnwrapObject
from convertURL import *


class TestConvertURL(unittest.TestCase):

    def __init__(self, foo):
        unittest.TestCase.__init__(self, foo)
        # we need to know the platform to determine which tests to run
        self.__infoService = components.classes["@activestate.com/koInfoService;1"] \
                       .getService(components.interfaces.koIInfoService)

    def test_LocalFileToURL(self):
        if self.__infoService.platform == "win32":
            url = "file:///C:/Documents%20and%20Settings/test/My%20Documents/somefile.txt"
            path = r"C:\Documents and Settings\test\My Documents\somefile.txt"
        else:
            url = "file:///home/test/somefile.txt"
            path = "/home/test/somefile.txt"

        assert LocalFileToURL(path) == url
 
    def test_URLToLocalFile(self):
        if self.__infoService.platform == "win32":
            url = "file:///C:/Documents%20and%20Settings/test/My%20Documents/somefile.txt"
            path = r"C:\Documents and Settings\test\My Documents\somefile.txt"
        else:
            url = "file:///home/test/somefile.txt"
            path = "/home/test/somefile.txt"

        assert URLToLocalFile(url).path == path


    def test_LocalPathFromURL(self):
        if self.__infoService.platform == "win32":
            url = "file:///C:/Documents%20and%20Settings/test/My%20Documents/somefile.txt"
            path = r"C:\Documents and Settings\test\My Documents\somefile.txt"
        else:
            url = "file:///home/test/somefile.txt"
            path = "/home/test/somefile.txt"

        assert LocalPathFromURL(url) == path

#    def test_RelativizeURL(self):


#---- mainline

def suite():
    return unittest.makeSuite(TestConvertURL)

def test_main():
    runner = unittest.TextTestRunner(verbosity=2)
    runner.run(suite())

if __name__ == "__main__":
    test_main()



