# test Komodo's startup cases

import os, sys, time
import unittest

import which # need which >=1.0.0 
if which._version_ < (1,0,0):
    raise ImportError("Need which v1.0.0 or greater. You can get it from:\n"
                      "\thttp://starship.python.net/~tmick/downloads/tmlib/")


#---- test cases

class StartupTestCase(unittest.TestCase):
    def _GetMozillaPIDs(self):
        if sys.platform.startswith("win"):
            import win32pdh, win32pdhutil
            win32pdh.EnumObjects(None, None, 0, 1)  # refresh internal cache
            pids = win32pdhutil.FindPerformanceAttributesByName("mozilla",
                                                                "Process",
                                                                "ID Process")
            return pids
        else:
            raise "Don't know how to count the mozilla processes yet."

    def _Kill(self, pid):
        if sys.platform.startswith("win"):
            import win32api, win32con
            handle = win32api.OpenProcess(win32con.PROCESS_TERMINATE, 0, pid)
            win32api.TerminateProcess(handle, 0)
            win32api.CloseHandle(handle)
        else:
            os.kill(pid, 9)

    def test_starts(self):
        """check that Komodo even starts"""
        # count the 'mozilla' processes
        mozillaPIDsBefore = self._GetMozillaPIDs()
        # spawn Komodo
        komodoExe = which.which("komodo")
        os.spawnv(os.P_NOWAIT, komodoExe, [komodoExe])
        # - XXX Should instead synchronize with Komodo startup. No mechanism
        #   for that yet, though.
        time.sleep(20)  # is this long enough?
        # the count of 'mozilla' processes should be one more
        mozillaPIDsAfter = self._GetMozillaPIDs()
        try:
            self.assertEqual(len(mozillaPIDsBefore)+1, len(mozillaPIDsAfter))
        finally:
            newPids = [pid for pid in mozillaPIDsAfter
                       if pid not in mozillaPIDsBefore]
            self._Kill(newPids[0])



#---- mainline

def suite():
    return unittest.makeSuite(StartupTestCase)

def test_main():
    runner = unittest.TextTestRunner(verbosity=2)
    runner.run(suite())

if __name__ == "__main__":
    test_main()


